@@ -3,7 +3,8 @@
 // Purpose:
 // Date: 08/11/1999
 // Author: Guilhem Lavaux <lavaux@easynet.fr> (C) 1999
-// CVSID: $Id: sndcodec.h,v 1.1 2000/03/05 19:03:12 GL Exp $
+// CVSID: $Id: sndcodec.h,v 1.2.4.1 2004/11/22 20:04:46 JS Exp $
+// License:    wxWindows license
 // --------------------------------------------------------------------------
 #ifndef _WX_SNDCODEC_H
 #define _WX_SNDCODEC_H
@@ -15,7 +16,7 @@
 #include "wx/defs.h"
 #include "wx/mmedia/sndbase.h"
 
-class wxSoundStreamCodec: public wxSoundStream {
+class WXDLLEXPORT wxSoundStreamCodec: public wxSoundStream {
  public:
   wxSoundStreamCodec(wxSoundStream& snd_io);
   ~wxSoundStreamCodec();
