@@ -3,7 +3,8 @@
 // Purpose:
 // Date: 08/11/1999
 // Author: Guilhem Lavaux <lavaux@easynet.fr> (C) 1999
-// CVSID: $Id: sndoss.h,v 1.1 2000/03/05 19:03:13 GL Exp $
+// CVSID: $Id: sndoss.h,v 1.2.4.1 2004/11/22 20:04:47 JS Exp $
+// License:    wxWindows license
 // --------------------------------------------------------------------------
 #ifndef _WX_SNDOSS_H
 #define _WX_SNDOSS_H
@@ -21,7 +22,7 @@
 // OSS output class
 //
 
-class wxSoundStreamOSS : public wxSoundStream {
+class WXDLLEXPORT wxSoundStreamOSS : public wxSoundStream {
  public:
   wxSoundStreamOSS(const wxString& dev_name = wxT("/dev/dsp"));
   ~wxSoundStreamOSS();
