@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     12/07/98
-// RCS-ID:      $Id: drawn.cpp,v 1.1 2000/03/03 11:25:04 JS Exp $
+// RCS-ID:      $Id: drawn.cpp,v 1.3.2.3 2002/12/18 06:13:00 RD Exp $
 // Copyright:   (c) Julian Smart
 // Licence:   	wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -15,7 +15,7 @@
 #endif
 
 // For compilers that support precompilation, includes "wx.h".
-#include <wx/wxprec.h>
+#include "wx/wxprec.h"
 
 #ifdef __BORLANDC__
 #pragma hdrstop
@@ -35,6 +35,8 @@
 #include <wx/ogl/drawnp.h>
 #include <wx/ogl/misc.h>
 
+#include <math.h>
+
 static void IntToHex(unsigned int dec, char *buf);
 static unsigned long HexToInt(char *buf);
 extern char *oglBuffer;
@@ -71,7 +73,7 @@ void wxDrawnShape::OnDraw(wxDC& dc)
     m_metafiles[m_currentAngle].m_outlinePen = g_oglTransparentPen;
     m_metafiles[m_currentAngle].Draw(dc, m_xpos + m_shadowOffsetX, m_ypos + m_shadowOffsetY);
   }
-    
+
   m_metafiles[m_currentAngle].m_outlinePen = m_pen;
   m_metafiles[m_currentAngle].m_fillBrush = m_brush;
   m_metafiles[m_currentAngle].Draw(dc, m_xpos, m_ypos);
@@ -137,7 +139,7 @@ void wxDrawnShape::Rotate(double x, double y, double theta)
     // Rotate metafile
     if (!m_metafiles[0].GetRotateable())
       return;
-    
+
     m_metafiles[0].Rotate(x, y, theta);
   }
 
@@ -234,7 +236,7 @@ bool wxDrawnShape::GetPerimeterPoint(double x1, double y1,
     return wxRectangleShape::GetPerimeterPoint(x1, y1, x2, y2, x3, y3);
 }
 
-#ifdef PROLOGIO
+#if wxUSE_PROLOGIO
 void wxDrawnShape::WriteAttributes(wxExpr *clause)
 {
   wxRectangleShape::WriteAttributes(clause);
@@ -290,7 +292,7 @@ void wxDrawnShape::Copy(wxShape& copy)
   drawnCopy.m_currentAngle = m_currentAngle;
 }
 
-bool wxDrawnShape::LoadFromMetaFile(char *filename)
+bool wxDrawnShape::LoadFromMetaFile(const wxString& filename)
 {
   return m_metafiles[0].LoadFromMetaFile(filename, &m_width, &m_height);
 }
@@ -406,12 +408,12 @@ void wxDrawnShape::SetDrawnBackgroundMode(int mode)
  * Individual operations
  *
  */
- 
+
 /*
  * Set font, brush, text colour
  *
  */
- 
+
 wxOpSetGDI::wxOpSetGDI(int theOp, wxPseudoMetaFile *theImage, int theGdiIndex, int theMode):
   wxDrawOp(theOp)
 {
@@ -520,6 +522,7 @@ wxDrawOp *wxOpSetGDI::Copy(wxPseudoMetaFile *newImage)
   return newOp;
 }
 
+#if wxUSE_PROLOGIO
 wxExpr *wxOpSetGDI::WriteExpr(wxPseudoMetaFile *image)
 {
   wxExpr *expr = new wxExpr(wxExprList);
@@ -580,12 +583,13 @@ void wxOpSetGDI::ReadExpr(wxPseudoMetaFile *image, wxExpr *expr)
       break;
   }
 }
+#endif
 
 /*
  * Set/destroy clipping
  *
  */
- 
+
 wxOpSetClipping::wxOpSetClipping(int theOp, double theX1, double theY1,
     double theX2, double theY2):wxDrawOp(theOp)
 {
@@ -600,7 +604,7 @@ wxDrawOp *wxOpSetClipping::Copy(wxPseudoMetaFile *newImage)
   wxOpSetClipping *newOp = new wxOpSetClipping(m_op, m_x1, m_y1, m_x2, m_y2);
   return newOp;
 }
-    
+
 void wxOpSetClipping::Do(wxDC& dc, double xoffset, double yoffset)
 {
   switch (m_op)
@@ -634,6 +638,7 @@ void wxOpSetClipping::Translate(double x, double y)
   m_y1 += y;
 }
 
+#if wxUSE_PROLOGIO
 wxExpr *wxOpSetClipping::WriteExpr(wxPseudoMetaFile *image)
 {
   wxExpr *expr = new wxExpr(wxExprList);
@@ -670,14 +675,15 @@ void wxOpSetClipping::ReadExpr(wxPseudoMetaFile *image, wxExpr *expr)
       break;
   }
 }
+#endif
 
 /*
  * Draw line, rectangle, rounded rectangle, ellipse, point, arc, text
  *
  */
- 
+
 wxOpDraw::wxOpDraw(int theOp, double theX1, double theY1, double theX2, double theY2,
-         double theRadius, char *s):wxDrawOp(theOp)
+         double theRadius, wxChar *s) : wxDrawOp(theOp)
 {
   m_x1 = theX1;
   m_y1 = theY1;
@@ -878,6 +884,7 @@ void wxOpDraw::Rotate(double x, double y, double theta, double sinTheta, double
   }
 }
 
+#if wxUSE_PROLOGIO
 wxExpr *wxOpDraw::WriteExpr(wxPseudoMetaFile *image)
 {
   wxExpr *expr = new wxExpr(wxExprList);
@@ -969,7 +976,7 @@ void wxOpDraw::ReadExpr(wxPseudoMetaFile *image, wxExpr *expr)
       m_x1 = expr->Nth(1)->RealValue();
       m_y1 = expr->Nth(2)->RealValue();
       wxString str(expr->Nth(3)->StringValue());
-      m_textString = copystring((const char*) str);
+      m_textString = copystring(str);
       break;
     }
     case DRAWOP_DRAW_ARC:
@@ -989,6 +996,7 @@ void wxOpDraw::ReadExpr(wxPseudoMetaFile *image, wxExpr *expr)
     }
   }
 }
+#endif
 
 /*
  * Draw polygon, polyline, spline
@@ -1102,6 +1110,7 @@ void wxOpPolyDraw::Rotate(double x, double y, double theta, double sinTheta, dou
   }
 }
 
+#if wxUSE_PROLOGIO
 wxExpr *wxOpPolyDraw::WriteExpr(wxPseudoMetaFile *image)
 {
   wxExpr *expr = new wxExpr(wxExprList);
@@ -1119,7 +1128,7 @@ wxExpr *wxOpPolyDraw::WriteExpr(wxPseudoMetaFile *image)
    * E.g. "1B9080CD". 4 hex digits per coordinate pair.
    *
    */
-   
+
   for (int i = 0; i < m_noPoints; i++)
   {
     long signedX = (long)(m_points[i].x*100.0);
@@ -1128,7 +1137,7 @@ wxExpr *wxOpPolyDraw::WriteExpr(wxPseudoMetaFile *image)
     // Scale to 0 -> 64K
     long unSignedX = (long)(signedX + 32767.0);
     long unSignedY = (long)(signedY + 32767.0);
-    
+
 //    IntToHex((unsigned int)signedX, buf2);
 //    IntToHex((unsigned int)signedY, buf3);
     IntToHex((int)unSignedX, buf2);
@@ -1163,7 +1172,7 @@ void wxOpPolyDraw::ReadExpr(wxPseudoMetaFile *image, wxExpr *expr)
     buf1[2] = hexString[(size_t)(bufPtr + 2)];
     buf1[3] = hexString[(size_t)(bufPtr + 3)];
     buf1[4] = 0;
-    
+
     buf2[0] = hexString[(size_t)(bufPtr + 4)];
     buf2[1] = hexString[(size_t)(bufPtr + 5)];
     buf2[2] = hexString[(size_t)(bufPtr + 6)];
@@ -1190,6 +1199,7 @@ void wxOpPolyDraw::ReadExpr(wxPseudoMetaFile *image, wxExpr *expr)
     i ++;
   }
 }
+#endif
 
 // Draw an outline using the current operation.
 bool wxOpPolyDraw::OnDrawOutline(wxDC& dc, double x, double y, double w, double h, double oldW, double oldH)
@@ -1251,7 +1261,7 @@ bool wxOpPolyDraw::GetPerimeterPoint(double x1, double y1,
       }
     }
   }
-  
+
   double *xpoints = new double[n];
   double *ypoints = new double[n];
 
@@ -1263,7 +1273,7 @@ bool wxOpPolyDraw::GetPerimeterPoint(double x1, double y1,
     ypoints[i] = point->y + yOffset;
   }
 
-  oglFindEndForPolyline(n, xpoints, ypoints, 
+  oglFindEndForPolyline(n, xpoints, ypoints,
                         x1, y1, x2, y2, x3, y3);
 
   delete[] xpoints;
@@ -1288,7 +1298,7 @@ static void IntToHex(unsigned int dec, char *buf)
   int digit2 = (int)((dec - (digit1*4096))/256);
   int digit3 = (int)((dec - (digit1*4096) - (digit2*256))/16);
   int digit4 = dec - (digit1*4096 + digit2*256 + digit3*16);
-  
+
   buf[0] = hexArray[digit1];
   buf[1] = hexArray[digit2];
   buf[2] = hexArray[digit3];
@@ -1446,20 +1456,20 @@ void wxPseudoMetaFile::Rotate(double x, double y, double theta)
   m_currentRotation = theta;
 }
 
-#ifdef PROLOGIO
+#if wxUSE_PROLOGIO
 void wxPseudoMetaFile::WriteAttributes(wxExpr *clause, int whichAngle)
 {
   wxString widthStr;
-  widthStr.Printf("meta_width%d", whichAngle);
+  widthStr.Printf(wxT("meta_width%d"), whichAngle);
 
   wxString heightStr;
-  heightStr.Printf("meta_height%d", whichAngle);
+  heightStr.Printf(wxT("meta_height%d"), whichAngle);
 
   wxString outlineStr;
-  outlineStr.Printf("outline_op%d", whichAngle);
+  outlineStr.Printf(wxT("outline_op%d"), whichAngle);
 
   wxString rotateableStr;
-  rotateableStr.Printf("meta_rotateable%d", whichAngle);
+  rotateableStr.Printf(wxT("meta_rotateable%d"), whichAngle);
 
   // Write width and height
   clause->AddAttributeValue(widthStr, m_width);
@@ -1553,7 +1563,7 @@ void wxPseudoMetaFile::WriteAttributes(wxExpr *clause, int whichAngle)
       node = node->Next();
     }
     wxString outlineObjectsStr;
-    outlineObjectsStr.Printf("outline_objects%d", whichAngle);
+    outlineObjectsStr.Printf(wxT("outline_objects%d"), whichAngle);
 
     clause->AddAttributeValue(outlineObjectsStr, outlineExpr);
   }
@@ -1567,26 +1577,26 @@ void wxPseudoMetaFile::WriteAttributes(wxExpr *clause, int whichAngle)
       node = node->Next();
     }
     wxString fillObjectsStr;
-    fillObjectsStr.Printf("fill_objects%d", whichAngle);
+    fillObjectsStr.Printf(wxT("fill_objects%d"), whichAngle);
 
     clause->AddAttributeValue(fillObjectsStr, fillExpr);
   }
-    
+
 }
 
 void wxPseudoMetaFile::ReadAttributes(wxExpr *clause, int whichAngle)
 {
   wxString widthStr;
-  widthStr.Printf("meta_width%d", whichAngle);
+  widthStr.Printf(wxT("meta_width%d"), whichAngle);
 
   wxString heightStr;
-  heightStr.Printf("meta_height%d", whichAngle);
+  heightStr.Printf(wxT("meta_height%d"), whichAngle);
 
   wxString outlineStr;
-  outlineStr.Printf("outline_op%d", whichAngle);
+  outlineStr.Printf(wxT("outline_op%d"), whichAngle);
 
   wxString rotateableStr;
-  rotateableStr.Printf("meta_rotateable%d", whichAngle);
+  rotateableStr.Printf(wxT("meta_rotateable%d"), whichAngle);
 
   clause->GetAttributeValue(widthStr, m_width);
   clause->GetAttributeValue(heightStr, m_height);
@@ -1693,7 +1703,7 @@ void wxPseudoMetaFile::ReadAttributes(wxExpr *clause, int whichAngle)
           m_ops.Append(theOp);
           break;
         }
-        
+
         case DRAWOP_SET_CLIPPING_RECT:
         case DRAWOP_DESTROY_CLIPPING_RECT:
         {
@@ -1733,7 +1743,7 @@ void wxPseudoMetaFile::ReadAttributes(wxExpr *clause, int whichAngle)
   }
 
   wxString outlineObjectsStr;
-  outlineObjectsStr.Printf("outline_objects%d", whichAngle);
+  outlineObjectsStr.Printf(wxT("outline_objects%d"), whichAngle);
 
   // Now read in the list of outline and fill operations, if any
   wxExpr *expr1 = clause->AttributeValue(outlineObjectsStr);
@@ -1748,7 +1758,7 @@ void wxPseudoMetaFile::ReadAttributes(wxExpr *clause, int whichAngle)
   }
 
   wxString fillObjectsStr;
-  fillObjectsStr.Printf("fill_objects%d", whichAngle);
+  fillObjectsStr.Printf(wxT("fill_objects%d"), whichAngle);
 
   expr1 = clause->AttributeValue(fillObjectsStr);
   if (expr1)
@@ -1784,7 +1794,7 @@ void wxPseudoMetaFile::Copy(wxPseudoMetaFile& copy)
     copy.m_gdiObjects.Append(obj);
     node = node->Next();
   }
-  
+
   // Copy the operations
   node = m_ops.First();
   while (node)
@@ -1814,14 +1824,14 @@ void wxPseudoMetaFile::Copy(wxPseudoMetaFile& copy)
  * fit width and return new width and height.
  *
  */
- 
-bool wxPseudoMetaFile::LoadFromMetaFile(char *filename, double *rwidth, double *rheight)
+
+bool wxPseudoMetaFile::LoadFromMetaFile(const wxString& filename, double *rwidth, double *rheight)
 {
-  if (!FileExists(filename))
+  if (!wxFileExists(filename))
     return NULL;
-    
+
   wxXMetaFile *metaFile = new wxXMetaFile;
-  
+
   if (!metaFile->ReadFile(filename))
   {
     delete metaFile;
@@ -1977,7 +1987,7 @@ bool wxPseudoMetaFile::LoadFromMetaFile(char *filename, double *rwidth, double *
           newPoints[i].x = record->points[i].x;
           newPoints[i].y = record->points[i].y;
         }
-        
+
         wxOpPolyDraw *op = new wxOpPolyDraw(DRAWOP_DRAW_POLYGON, n, newPoints);
         m_ops.Append(op);
         break;
@@ -1991,7 +2001,7 @@ bool wxPseudoMetaFile::LoadFromMetaFile(char *filename, double *rwidth, double *
           newPoints[i].x = record->points[i].x;
           newPoints[i].y = record->points[i].y;
         }
-        
+
         wxOpPolyDraw *op = new wxOpPolyDraw(DRAWOP_DRAW_POLYLINE, n, newPoints);
         m_ops.Append(op);
         break;
