@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     12/07/98
-// RCS-ID:      $Id: lines.cpp,v 1.1 2000/03/03 11:25:04 JS Exp $
+// RCS-ID:      $Id: lines.cpp,v 1.6.2.6 2003/12/11 06:30:50 JS Exp $
 // Copyright:   (c) Julian Smart
 // Licence:   	wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -15,7 +15,7 @@
 #endif
 
 // For compilers that support precompilation, includes "wx.h".
-#include <wx/wxprec.h>
+#include "wx/wxprec.h"
 
 #ifdef __BORLANDC__
 #pragma hdrstop
@@ -27,10 +27,8 @@
 
 #include <wx/wxexpr.h>
 
-#if wxUSE_IOSTREAMH
-#include <iostream.h>
-#else
-#include <iostream>
+#ifdef new
+#undef new
 #endif
 
 #include <ctype.h>
@@ -73,17 +71,17 @@ wxLineShape::wxLineShape()
   // and make the three line regions.
   ClearRegions();
   wxShapeRegion *newRegion = new wxShapeRegion;
-  newRegion->SetName("Middle");
+  newRegion->SetName(wxT("Middle"));
   newRegion->SetSize(150, 50);
   m_regions.Append((wxObject *)newRegion);
 
   newRegion = new wxShapeRegion;
-  newRegion->SetName("Start");
+  newRegion->SetName(wxT("Start"));
   newRegion->SetSize(150, 50);
   m_regions.Append((wxObject *)newRegion);
 
   newRegion = new wxShapeRegion;
-  newRegion->SetName("End");
+  newRegion->SetName(wxT("End"));
   newRegion->SetSize(150, 50);
   m_regions.Append((wxObject *)newRegion);
 
@@ -220,7 +218,7 @@ void wxLineShape::FormatText(wxDC& dc, const wxString& s, int i)
 
   region->GetSize(&w, &h);
   // Initialize the size if zero
-  if (((w == 0) || (h == 0)) && (strlen(s) > 0))
+  if (((w == 0) || (h == 0)) && (s.Length() > 0))
   {
     w = 100; h = 50;
     region->SetSize(w, h);
@@ -230,7 +228,7 @@ void wxLineShape::FormatText(wxDC& dc, const wxString& s, int i)
   node = string_list->First();
   while (node)
   {
-    char *s = (char *)node->Data();
+    wxChar *s = (wxChar *)node->Data();
     wxShapeTextLine *line = new wxShapeTextLine(0.0, 0.0, s);
     region->GetFormattedText().Append((wxObject *)line);
     node = node->Next();
@@ -284,8 +282,8 @@ void wxLineShape::DrawRegion(wxDC& dc, wxShapeRegion *region, double x, double y
   // First, clear a rectangle for the text IF there is any
   if (region->GetFormattedText().Number() > 0)
   {
-      dc.SetPen(* g_oglWhiteBackgroundPen);
-      dc.SetBrush(* g_oglWhiteBackgroundBrush);
+      dc.SetPen(GetBackgroundPen());
+      dc.SetBrush(GetBackgroundBrush());
 
       // Now draw the text
       if (region->GetFont()) dc.SetFont(* region->GetFont());
@@ -296,7 +294,7 @@ void wxLineShape::DrawRegion(wxDC& dc, wxShapeRegion *region, double x, double y
       dc.SetTextForeground(* region->GetActualColourObject());
 
 #ifdef __WXMSW__
-      dc.SetTextBackground(g_oglWhiteBackgroundBrush->GetColour());
+      dc.SetTextBackground(GetBackgroundBrush().GetColour());
 #endif
 
       oglDrawFormattedText(dc, &(region->GetFormattedText()), xp, yp, w, h, region->GetFormatMode());
@@ -320,8 +318,8 @@ void wxLineShape::EraseRegion(wxDC& dc, wxShapeRegion *region, double x, double
 
   if (region->GetFormattedText().Number() > 0)
   {
-      dc.SetPen(* g_oglWhiteBackgroundPen);
-      dc.SetBrush(* g_oglWhiteBackgroundBrush);
+      dc.SetPen(GetBackgroundPen());
+      dc.SetBrush(GetBackgroundBrush());
 
       dc.DrawRectangle((long)(xp - w/2.0), (long)(yp - h/2.0), (long)w, (long)h);
   }
@@ -509,22 +507,23 @@ bool wxLineShape::HitTest(double x, double y, int *attachment, double *distance)
     wxRealPoint *point1 = (wxRealPoint *)node->Data();
     wxRealPoint *point2 = (wxRealPoint *)node->Next()->Data();
 
-    // Allow for inaccurate mousing or vert/horiz lines
+    // For inaccurate mousing allow 8 pixel corridor
     int extra = 4;
-    double left = wxMin(point1->x, point2->x) - extra;
-    double right = wxMax(point1->x, point2->x) + extra;
 
-    double bottom = wxMin(point1->y, point2->y) - extra;
-    double top = wxMax(point1->y, point2->y) + extra;
-
-    if ((x > left && x < right && y > bottom && y < top) || inLabelRegion)
+    double dx = point2->x - point1->x;
+    double dy = point2->y - point1->y;
+    double seg_len = sqrt(dx*dx+dy*dy);
+    double distance_from_seg =
+      seg_len*((x-point1->x)*dy-(y-point1->y)*dx)/(dy*dy+dx*dx);
+    double distance_from_prev =
+      seg_len*((y-point1->y)*dy+(x-point1->x)*dx)/(dy*dy+dx*dx);
+    
+    if ((fabs(distance_from_seg) < extra &&
+         distance_from_prev >= 0 && distance_from_prev <= seg_len)
+        || inLabelRegion)
     {
-      // Work out distance from centre of line
-      double centre_x = (double)(left + (right - left)/2.0);
-      double centre_y = (double)(bottom + (top - bottom)/2.0);
-
       *attachment = 0;
-      *distance = (double)sqrt((centre_x - x)*(centre_x - x) + (centre_y - y)*(centre_y - y));
+      *distance = distance_from_seg;
       return TRUE;
     }
 
@@ -750,7 +749,7 @@ void wxLineShape::DrawArrow(wxDC& dc, wxArrowHead *arrow, double xOffset, bool p
 
       dc.SetPen(* m_pen);
       if (arrow->_GetType() == ARROW_HOLLOW_CIRCLE)
-        dc.SetBrush(* g_oglWhiteBackgroundBrush);
+        dc.SetBrush(GetBackgroundBrush());
       else
         dc.SetBrush(* m_brush);
 
@@ -816,7 +815,7 @@ void wxLineShape::DrawArrow(wxDC& dc, wxArrowHead *arrow, double xOffset, bool p
 
         else
         {
-          wxFatalError("Unknown arrowhead rotation case in lines.cc");
+          wxLogFatalError(wxT("Unknown arrowhead rotation case in lines.cc"));
         }
 
         // Rotate about the centre of the object, then place
@@ -849,8 +848,10 @@ void wxLineShape::OnErase(wxDC& dc)
 {
     wxPen *old_pen = m_pen;
     wxBrush *old_brush = m_brush;
-    SetPen(g_oglWhiteBackgroundPen);
-    SetBrush(g_oglWhiteBackgroundBrush);
+    wxPen bg_pen = GetBackgroundPen();
+    wxBrush bg_brush = GetBackgroundBrush();
+    SetPen(&bg_pen);
+    SetBrush(&bg_brush);
 
     double bound_x, bound_y;
     GetBoundingBoxMax(&bound_x, &bound_y);
@@ -870,8 +871,8 @@ void wxLineShape::OnErase(wxDC& dc)
     }
 
     // Undraw line
-    dc.SetPen(* g_oglWhiteBackgroundPen);
-    dc.SetBrush(* g_oglWhiteBackgroundBrush);
+    dc.SetPen(GetBackgroundPen());
+    dc.SetBrush(GetBackgroundBrush());
 
     // Drawing over the line only seems to work if the line has a thickness
     // of 1.
@@ -1367,7 +1368,7 @@ void wxLineShape::ResetControlPoints()
   }
 }
 
-#ifdef PROLOGIO
+#if wxUSE_PROLOGIO
 void wxLineShape::WriteAttributes(wxExpr *clause)
 {
   wxShape::WriteAttributes(clause);
@@ -1443,15 +1444,15 @@ void wxLineShape::ReadAttributes(wxExpr *clause)
   wxShape::ReadAttributes(clause);
 
   int iVal = (int) m_isSpline;
-  clause->AssignAttributeValue("is_spline", &iVal);
+  clause->AssignAttributeValue(wxT("is_spline"), &iVal);
   m_isSpline = (iVal != 0);
 
   iVal = (int) m_maintainStraightLines;
-  clause->AssignAttributeValue("keep_lines_straight", &iVal);
+  clause->AssignAttributeValue(wxT("keep_lines_straight"), &iVal);
   m_maintainStraightLines = (iVal != 0);
 
-  clause->AssignAttributeValue("align_start", &m_alignmentStart);
-  clause->AssignAttributeValue("align_end", &m_alignmentEnd);
+  clause->AssignAttributeValue(wxT("align_start"), &m_alignmentStart);
+  clause->AssignAttributeValue(wxT("align_end"), &m_alignmentEnd);
 
   // Compatibility: check for no regions.
   if (m_regions.Number() == 0)
@@ -1475,12 +1476,12 @@ void wxLineShape::ReadAttributes(wxExpr *clause)
     }
 
     newRegion = new wxShapeRegion;
-    newRegion->SetName("Start");
+    newRegion->SetName(wxT("Start"));
     newRegion->SetSize(150, 50);
     m_regions.Append((wxObject *)newRegion);
 
     newRegion = new wxShapeRegion;
-    newRegion->SetName("End");
+    newRegion->SetName(wxT("End"));
     newRegion->SetSize(150, 50);
     m_regions.Append((wxObject *)newRegion);
   }
@@ -1488,14 +1489,14 @@ void wxLineShape::ReadAttributes(wxExpr *clause)
   m_attachmentTo = 0;
   m_attachmentFrom = 0;
 
-  clause->AssignAttributeValue("attachment_to", &m_attachmentTo);
-  clause->AssignAttributeValue("attachment_from", &m_attachmentFrom);
+  clause->AssignAttributeValue(wxT("attachment_to"), &m_attachmentTo);
+  clause->AssignAttributeValue(wxT("attachment_from"), &m_attachmentFrom);
 
   wxExpr *line_list = NULL;
 
   // When image is created, there are default control points. Override
   // them if there are some in the file.
-  clause->AssignAttributeValue("controls", &line_list);
+  clause->AssignAttributeValue(wxT("controls"), &line_list);
 
   if (line_list)
   {
@@ -1527,7 +1528,7 @@ void wxLineShape::ReadAttributes(wxExpr *clause)
   // Read arrow list, for new OGL code
   wxExpr *arrow_list = NULL;
 
-  clause->AssignAttributeValue("arrows", &arrow_list);
+  clause->AssignAttributeValue(wxT("arrows"), &arrow_list);
   if (arrow_list)
   {
     wxExpr *node = arrow_list->value.first;
@@ -1538,7 +1539,7 @@ void wxLineShape::ReadAttributes(wxExpr *clause)
       int arrowEnd = 0;
       double xOffset = 0.0;
       double arrowSize = 0.0;
-      wxString arrowName("");
+      wxString arrowName;
       long arrowId = -1;
 
       wxExpr *type_expr = node->Nth(0);
@@ -1570,7 +1571,7 @@ void wxLineShape::ReadAttributes(wxExpr *clause)
       else
         wxRegisterId(arrowId);
 
-      wxArrowHead *arrowHead = AddArrow(arrowType, arrowEnd, arrowSize, xOffset, (char*) (const char*) arrowName, NULL, arrowId);
+      wxArrowHead *arrowHead = AddArrow(arrowType, arrowEnd, arrowSize, xOffset, arrowName, NULL, arrowId);
       if (yOffsetExpr)
         arrowHead->SetYOffset(yOffsetExpr->RealValue());
       if (spacingExpr)
@@ -1813,7 +1814,7 @@ void wxLineShape::OnSizingBeginDragLeft(wxControlPoint* pt, double x, double y,
 
   if (lpt->m_type == CONTROL_POINT_ENDPOINT_FROM || lpt->m_type == CONTROL_POINT_ENDPOINT_TO)
   {
-    m_canvas->SetCursor(* g_oglBullseyeCursor);
+    m_canvas->SetCursor(wxCursor(wxCURSOR_BULLSEYE));
     lpt->m_oldCursor = wxSTANDARD_CURSOR;
   }
 }
@@ -1929,7 +1930,7 @@ void wxLineControlPoint::OnBeginDragRight(double x, double y, int keys, int atta
       lineShape->GetTo()->GetEventHandler()->OnDraw(dc);
       lineShape->GetTo()->GetEventHandler()->OnDrawContents(dc);
     }
-    m_canvas->SetCursor(g_oglBullseyeCursor);
+    m_canvas->SetCursor(wxCursor(wxCURSOR_BULLSEYE));
     m_oldCursor = wxSTANDARD_CURSOR;
   }
 }
@@ -2334,7 +2335,7 @@ wxRealPoint *wxLineShape::GetNextControlPoint(wxShape *nodeObject)
     return (wxRealPoint *)node->Data();
   }
   else
-    return FALSE;
+    return NULL;
 }
 
 /*
