@@ -4,15 +4,15 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     01/02/97
-// RCS-ID:      $Id: docview.h,v 1.32 2000/03/20 13:41:16 JS Exp $
-// Copyright:   (c)
+// RCS-ID:      $Id: docview.h,v 1.45.2.3 2005/05/04 18:23:26 JS Exp $
+// Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
 #ifndef _WX_DOCH__
 #define _WX_DOCH__
 
-#ifdef __GNUG__
+#if defined(__GNUG__) && !defined(__APPLE__)
     #pragma interface "docview.h"
 #endif
 
@@ -20,6 +20,7 @@
 #include "wx/list.h"
 #include "wx/cmndata.h"
 #include "wx/string.h"
+#include "wx/frame.h"
 
 #if wxUSE_PRINTING_ARCHITECTURE
     #include "wx/print.h"
@@ -31,7 +32,6 @@ class WXDLLEXPORT wxView;
 class WXDLLEXPORT wxDocTemplate;
 class WXDLLEXPORT wxDocManager;
 class WXDLLEXPORT wxPrintInfo;
-class WXDLLEXPORT wxCommand;
 class WXDLLEXPORT wxCommandProcessor;
 class WXDLLEXPORT wxFileHistory;
 class WXDLLEXPORT wxConfigBase;
@@ -64,8 +64,6 @@ enum
 
 class WXDLLEXPORT wxDocument : public wxEvtHandler
 {
-    DECLARE_ABSTRACT_CLASS(wxDocument)
-
 public:
     wxDocument(wxDocument *parent = (wxDocument *) NULL);
     ~wxDocument();
@@ -89,18 +87,13 @@ public:
     virtual bool Revert();
 
 #if wxUSE_STD_IOSTREAM
-    virtual ostream& SaveObject(ostream& stream);
-    virtual istream& LoadObject(istream& stream);
+    virtual wxSTD ostream& SaveObject(wxSTD ostream& stream);
+    virtual wxSTD istream& LoadObject(wxSTD istream& stream);
 #else
     virtual wxOutputStream& SaveObject(wxOutputStream& stream);
     virtual wxInputStream& LoadObject(wxInputStream& stream);
 #endif
 
-#if wxUSE_SERIAL
-    // need this to keep from hiding the virtual from wxObject
-    virtual void LoadObject(wxObjectInputStream& stream) { wxObject::LoadObject(stream); };
-#endif
-
     // Called by wxWindows
     virtual bool OnSaveDocument(const wxString& filename);
     virtual bool OnOpenDocument(const wxString& filename);
@@ -138,6 +131,7 @@ public:
     wxView *GetFirstView() const;
 
     virtual void UpdateAllViews(wxView *sender = (wxView *) NULL, wxObject *hint = (wxObject *) NULL);
+    virtual void NotifyClosing();
 
     // Remove all views (because we're closing the document)
     virtual bool DeleteAllViews();
@@ -164,30 +158,32 @@ protected:
     wxDocument*           m_documentParent;
     wxCommandProcessor*   m_commandProcessor;
     bool                  m_savedYet;
+    
+private:
+    DECLARE_ABSTRACT_CLASS(wxDocument)
 };
 
 class WXDLLEXPORT wxView: public wxEvtHandler
 {
-    DECLARE_ABSTRACT_CLASS(wxView)
-
 public:
     //  wxView(wxDocument *doc = (wxDocument *) NULL);
     wxView();
     ~wxView();
 
     wxDocument *GetDocument() const { return m_viewDocument; }
-    void SetDocument(wxDocument *doc);
+    virtual void SetDocument(wxDocument *doc);
 
     wxString GetViewName() const { return m_viewTypeName; }
     void SetViewName(const wxString& name) { m_viewTypeName = name; };
 
-    wxFrame *GetFrame() const { return m_viewFrame ; }
-    void SetFrame(wxFrame *frame) { m_viewFrame = frame; }
+    wxWindow *GetFrame() const { return m_viewFrame ; }
+    void SetFrame(wxWindow *frame) { m_viewFrame = frame; }
 
     virtual void OnActivateView(bool activate, wxView *activeView, wxView *deactiveView);
     virtual void OnDraw(wxDC *dc) = 0;
     virtual void OnPrint(wxDC *dc, wxObject *info);
     virtual void OnUpdate(wxView *sender, wxObject *hint = (wxObject *) NULL);
+    virtual void OnClosingDocument() {};
     virtual void OnChangeFilename();
 
     // Called by framework if created automatically by the default document
@@ -224,13 +220,15 @@ public:
 protected:
     wxDocument*       m_viewDocument;
     wxString          m_viewTypeName;
-    wxFrame*          m_viewFrame;
+    wxWindow*         m_viewFrame;
+    
+private:
+    DECLARE_ABSTRACT_CLASS(wxView)
 };
 
 // Represents user interface (and other) properties of documents and views
 class WXDLLEXPORT wxDocTemplate: public wxObject
 {
-DECLARE_CLASS(wxDocTemplate)
 
 friend class WXDLLEXPORT wxDocManager;
 
@@ -289,6 +287,9 @@ protected:
     // For dynamic creation of appropriate instances.
     wxClassInfo*      m_docClassInfo;
     wxClassInfo*      m_viewClassInfo;
+    
+private:
+    DECLARE_CLASS(wxDocTemplate)
 };
 
 // One object of this class may be created in an application, to manage all
@@ -305,6 +306,7 @@ public:
 
     // Handlers for common user commands
     void OnFileClose(wxCommandEvent& event);
+    void OnFileCloseAll(wxCommandEvent& event);
     void OnFileNew(wxCommandEvent& event);
     void OnFileOpen(wxCommandEvent& event);
     void OnFileRevert(wxCommandEvent& event);
@@ -335,9 +337,6 @@ public:
 
     // called when file format detection didn't work, can be overridden to do
     // something in this case
-    // This is of course completely stupid, because if the file dialog is
-    // cancelled you get an assert. Brilliant. -- JACS
-//    virtual void OnOpenFileFailure() { wxFAIL_MSG(_T("file format mismatch")); }
     virtual void OnOpenFileFailure() { }
 
     virtual wxDocument *CreateDocument(const wxString& path, long flags = 0);
@@ -348,9 +347,9 @@ public:
     virtual wxDocTemplate *SelectDocumentPath(wxDocTemplate **templates,
             int noTemplates, wxString& path, long flags, bool save = FALSE);
     virtual wxDocTemplate *SelectDocumentType(wxDocTemplate **templates,
-            int noTemplates);
+            int noTemplates, bool sort = FALSE);
     virtual wxDocTemplate *SelectViewType(wxDocTemplate **templates,
-            int noTemplates);
+            int noTemplates, bool sort = FALSE);
     virtual wxDocTemplate *FindTemplateForPath(const wxString& path);
 
     void AssociateTemplate(wxDocTemplate *temp);
@@ -365,6 +364,9 @@ public:
     void AddDocument(wxDocument *doc);
     void RemoveDocument(wxDocument *doc);
 
+    // closes all currently open documents
+    bool CloseDocuments(bool force = TRUE);
+
     // Clear remaining documents and templates
     bool Clear(bool force = TRUE);
 
@@ -373,7 +375,8 @@ public:
     virtual void ActivateView(wxView *view, bool activate = TRUE, bool deleting = FALSE);
     virtual wxView *GetCurrentView() const;
 
-    virtual wxList& GetDocuments() const { return (wxList&) m_docs; }
+    wxList& GetDocuments() { return m_docs; }
+    wxList& GetTemplates() { return m_templates; }
 
     // Make a default document name
     virtual bool MakeDefaultName(wxString& buf);
@@ -425,8 +428,6 @@ protected:
 
 class WXDLLEXPORT wxDocChildFrame : public wxFrame
 {
-    DECLARE_CLASS(wxDocChildFrame)
-
 public:
     wxDocChildFrame(wxDocument *doc,
                     wxView *view,
@@ -436,7 +437,7 @@ public:
                     const wxPoint& pos = wxDefaultPosition,
                     const wxSize& size = wxDefaultSize,
                     long type = wxDEFAULT_FRAME_STYLE,
-                    const wxString& name = "frame");
+                    const wxString& name = wxT("frame"));
     ~wxDocChildFrame();
 
     // Extend event processing to search the view's event table
@@ -449,11 +450,14 @@ public:
     wxView *GetView() const { return m_childView; }
     void SetDocument(wxDocument *doc) { m_childDocument = doc; }
     void SetView(wxView *view) { m_childView = view; }
+    bool Destroy() { m_childView = (wxView *)NULL; return wxFrame::Destroy(); }
 
 protected:
     wxDocument*       m_childDocument;
     wxView*           m_childView;
 
+private:
+    DECLARE_CLASS(wxDocChildFrame)
     DECLARE_EVENT_TABLE()
 };
 
@@ -463,8 +467,6 @@ protected:
 
 class WXDLLEXPORT wxDocParentFrame : public wxFrame
 {
-    DECLARE_CLASS(wxDocParentFrame)
-
 public:
     wxDocParentFrame(wxDocManager *manager,
                      wxFrame *frame,
@@ -473,7 +475,7 @@ public:
                      const wxPoint& pos = wxDefaultPosition,
                      const wxSize& size = wxDefaultSize,
                      long type = wxDEFAULT_FRAME_STYLE,
-                     const wxString& name = "frame");
+                     const wxString& name = wxT("frame"));
 
     // Extend event processing to search the document manager's event table
     virtual bool ProcessEvent(wxEvent& event);
@@ -487,6 +489,8 @@ public:
 protected:
     wxDocManager *m_docManager;
 
+private:
+    DECLARE_CLASS(wxDocParentFrame)
     DECLARE_EVENT_TABLE()
 };
 
@@ -497,10 +501,8 @@ protected:
 #if wxUSE_PRINTING_ARCHITECTURE
 class WXDLLEXPORT wxDocPrintout : public wxPrintout
 {
-    DECLARE_DYNAMIC_CLASS(wxDocPrintout)
-
 public:
-    wxDocPrintout(wxView *view = (wxView *) NULL, const wxString& title = "Printout");
+    wxDocPrintout(wxView *view = (wxView *) NULL, const wxString& title = wxT("Printout"));
     bool OnPrintPage(int page);
     bool HasPage(int page);
     bool OnBeginDocument(int startPage, int endPage);
@@ -510,76 +512,18 @@ public:
 
 protected:
     wxView*       m_printoutView;
+
+private:
+    DECLARE_DYNAMIC_CLASS(wxDocPrintout)
 };
 #endif // wxUSE_PRINTING_ARCHITECTURE
 
-// ----------------------------------------------------------------------------
-// Command processing framework
-// ----------------------------------------------------------------------------
-
-class WXDLLEXPORT wxCommand : public wxObject
-{
-    DECLARE_CLASS(wxCommand)
-
-public:
-    wxCommand(bool canUndoIt = FALSE, const wxString& name = "");
-    ~wxCommand();
-
-    // Override this to perform a command
-    virtual bool Do() = 0;
-
-    // Override this to undo a command
-    virtual bool Undo() = 0;
-
-    virtual bool CanUndo() const { return m_canUndo; }
-    virtual wxString GetName() const { return m_commandName; }
-
-protected:
-    bool     m_canUndo;
-    wxString m_commandName;
-};
-
-class WXDLLEXPORT wxCommandProcessor : public wxObject
-{
-    DECLARE_DYNAMIC_CLASS(wxCommandProcessor)
-
-public:
-    wxCommandProcessor(int maxCommands = 100);
-    ~wxCommandProcessor();
-
-    // Pass a command to the processor. The processor calls Do(); if
-    // successful, is appended to the command history unless storeIt is FALSE.
-    virtual bool Submit(wxCommand *command, bool storeIt = TRUE);
-    virtual bool Undo();
-    virtual bool Redo();
-    virtual bool CanUndo() const;
-    virtual bool CanRedo() const;
-
-    // Call this to manage an edit menu.
-    void SetEditMenu(wxMenu *menu) { m_commandEditMenu = menu; }
-    wxMenu *GetEditMenu() const { return m_commandEditMenu; }
-    virtual void SetMenuStrings();
-    virtual void Initialize();
-
-    wxList& GetCommands() const { return (wxList&) m_commands; }
-    int GetMaxCommands() const { return m_maxNoCommands; }
-    virtual void ClearCommands();
-
-protected:
-    int           m_maxNoCommands;
-    wxList        m_commands;
-    wxNode*       m_currentCommand;
-    wxMenu*       m_commandEditMenu;
-};
-
 // ----------------------------------------------------------------------------
 // File history management
 // ----------------------------------------------------------------------------
 
 class WXDLLEXPORT wxFileHistory : public wxObject
 {
-    DECLARE_DYNAMIC_CLASS(wxFileHistory)
-
 public:
     wxFileHistory(int maxFiles = 9);
     ~wxFileHistory();
@@ -619,13 +563,16 @@ protected:
     wxList            m_fileMenus;
     // Max files to maintain
     int               m_fileMaxFiles;
+    
+private:
+    DECLARE_DYNAMIC_CLASS(wxFileHistory)
 };
 
 #if wxUSE_STD_IOSTREAM
 // For compatibility with existing file formats:
 // converts from/to a stream to/from a temporary file.
-bool WXDLLEXPORT wxTransferFileToStream(const wxString& filename, ostream& stream);
-bool WXDLLEXPORT wxTransferStreamToFile(istream& stream, const wxString& filename);
+bool WXDLLEXPORT wxTransferFileToStream(const wxString& filename, wxSTD ostream& stream);
+bool WXDLLEXPORT wxTransferStreamToFile(wxSTD istream& stream, const wxString& filename);
 #else
 // For compatibility with existing file formats:
 // converts from/to a stream to/from a temporary file.
