@@ -4,15 +4,15 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     01/02/97
-// RCS-ID:      $Id: msgdlgg.h,v 1.10 2000/01/05 15:35:01 VZ Exp $
-// Copyright:   (c)
+// RCS-ID:      $Id: msgdlgg.h,v 1.13.2.1 2005/05/04 18:23:53 JS Exp $
+// Copyright:   (c) Julian Smart
 // Licence:   	wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
 #ifndef __MSGDLGH_G__
 #define __MSGDLGH_G__
 
-#ifdef __GNUG__
+#if defined(__GNUG__) && !defined(__APPLE__)
 #pragma interface "msgdlgg.h"
 #endif
 
@@ -43,17 +43,8 @@ private:
     DECLARE_EVENT_TABLE()
 };
 
-#if !defined( __WXMSW__ ) && !defined( __WXMAC__) && !defined(__WXPM__)
+#if (!defined( __WXMSW__ ) && !defined( __WXMAC__) && !defined(__WXPM__)) || defined(__WXUNIVERSAL__)
 #define wxMessageDialog wxGenericMessageDialog
-
-int wxMessageBox( const wxString& message
-                 ,const wxString& caption = wxMessageBoxCaptionStr
-                 ,long  style = wxOK|wxCENTRE
-                 ,wxWindow *parent = (wxWindow *) NULL
-                 ,int x = -1
-                 ,int y = -1
-                );
-
 #endif
 
 #endif
