@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by: Vadim Zeitlin to be less MSW-specific on 10/10/98
 // Created:     01/02/97
-// RCS-ID:      $Id: treectrl.h,v 1.44 2000/02/29 01:47:33 VZ Exp $
+// RCS-ID:      $Id: treectrl.h,v 1.51.2.5 2002/12/29 15:42:10 RL Exp $
 // Copyright:   (c) Julian Smart and Markus Holzem
 // Licence:     wxWindows license
 /////////////////////////////////////////////////////////////////////////////
@@ -20,8 +20,11 @@
     #pragma interface "treectrl.h"
 #endif
 
+#if wxUSE_TREECTRL
+
 #include "wx/textctrl.h"
 #include "wx/dynarray.h"
+#include "wx/treebase.h"
 
 #ifdef __GNUWIN32__
     // Cygwin windows.h defines these identifiers
@@ -58,68 +61,6 @@ enum
 #define wxTREE_INSERT_FIRST 0xFFFF0001
 #define wxTREE_INSERT_LAST  0xFFFF0002
 
-// ----------------------------------------------------------------------------
-// wxTreeItemId identifies an element of the tree. In this implementation, it's
-// just a trivial wrapper around Win32 HTREEITEM. It's opaque for the
-// application.
-// ----------------------------------------------------------------------------
-class WXDLLEXPORT wxTreeItemId
-{
-public:
-    // ctors
-        // 0 is invalid value for HTREEITEM
-    wxTreeItemId() { m_itemId = 0; }
-
-        // default copy ctor/assignment operator are ok for us
-
-    // accessors
-        // is this a valid tree item?
-    bool IsOk() const { return m_itemId != 0; }
-
-    // conversion to/from either real (system-dependent) tree item id or
-    // to "long" which used to be the type for tree item ids in previous
-    // versions of wxWindows
-
-    // for wxTreeCtrl usage only
-    wxTreeItemId(WXHTREEITEM itemId) { m_itemId = (long)itemId; }
-    operator WXHTREEITEM() const { return (WXHTREEITEM)m_itemId; }
-
-    void operator=(WXHTREEITEM item) { m_itemId = (long) item; }
-
-protected:
-    long m_itemId;
-};
-
-WX_DEFINE_EXPORTED_ARRAY(WXHTREEITEM, wxArrayTreeItemIds);
-
-// ----------------------------------------------------------------------------
-// wxTreeItemData is some (arbitrary) user class associated with some item. The
-// main advantage of having this class (compared to old untyped interface) is
-// that wxTreeItemData's are destroyed automatically by the tree and, as this
-// class has virtual dtor, it means that the memory will be automatically
-// freed. OTOH, we don't just use wxObject instead of wxTreeItemData because
-// the size of this class is critical: in any real application, each tree leaf
-// will have wxTreeItemData associated with it and number of leaves may be
-// quite big.
-//
-// Because the objects of this class are deleted by the tree, they should
-// always be allocated on the heap!
-// ----------------------------------------------------------------------------
-class WXDLLEXPORT wxTreeItemData : private wxTreeItemId
-{
-public:
-    // default ctor/copy ctor/assignment operator are ok
-
-    // dtor is virtual and all the items are deleted by the tree control when
-    // it's deleted, so you normally don't have to care about freeing memory
-    // allocated in your wxTreeItemData-derived class
-    virtual ~wxTreeItemData() { }
-
-    // accessors: set/get the item associated with this node
-    void SetId(const wxTreeItemId& id) { m_itemId = id; }
-    const wxTreeItemId GetId() const { return *this; }
-};
-
 // ----------------------------------------------------------------------------
 // wxTreeCtrl
 // ----------------------------------------------------------------------------
@@ -135,7 +76,7 @@ public:
                const wxSize& size = wxDefaultSize,
                long style = wxTR_HAS_BUTTONS | wxTR_LINES_AT_ROOT,
                const wxValidator& validator = wxDefaultValidator,
-               const wxString& name = "wxTreeCtrl")
+               const wxString& name = wxTreeCtrlNameStr)
     {
         Create(parent, id, pos, size, style, validator, name);
     }
@@ -147,7 +88,7 @@ public:
                 const wxSize& size = wxDefaultSize,
                 long style = wxTR_HAS_BUTTONS | wxTR_LINES_AT_ROOT,
                 const wxValidator& validator = wxDefaultValidator,
-                const wxString& name = "wxTreeCtrl");
+                const wxString& name = wxTreeCtrlNameStr);
 
     // accessors
     // ---------
@@ -181,6 +122,8 @@ public:
 
     void SetImageList(wxImageList *imageList);
     void SetStateImageList(wxImageList *imageList);
+    void AssignImageList(wxImageList *imageList);
+    void AssignStateImageList(wxImageList *imageList);
 
     // Functions to work with tree ctrl items. Unfortunately, they can _not_ be
     // member functions of wxTreeItem because they must know the tree the item
@@ -198,6 +141,15 @@ public:
         // get the data associated with the item
     wxTreeItemData *GetItemData(const wxTreeItemId& item) const;
 
+        // get the item's text colour
+    wxColour GetItemTextColour(const wxTreeItemId& item) const;
+
+        // get the item's background colour
+    wxColour GetItemBackgroundColour(const wxTreeItemId& item) const;
+
+        // get the item's font
+    wxFont GetItemFont(const wxTreeItemId& item) const;
+
     // modifiers
     // ---------
 
@@ -270,7 +222,16 @@ public:
     size_t GetSelections(wxArrayTreeItemIds& selections) const;
 
         // get the parent of this item (may return NULL if root)
-    wxTreeItemId GetParent(const wxTreeItemId& item) const;
+    wxTreeItemId GetItemParent(const wxTreeItemId& item) const;
+
+#if WXWIN_COMPATIBILITY_2_2
+        // deprecated:  Use GetItemParent instead.
+    wxTreeItemId GetParent(const wxTreeItemId& item) const
+    	{ return GetItemParent( item ); }
+
+    	// Expose the base class method hidden by the one above.
+    wxWindow *GetParent() const { return wxControl::GetParent(); }
+#endif  // WXWIN_COMPATIBILITY_2_2
 
         // for this enumeration function you must pass in a "cookie" parameter
         // which is opaque for the application but is necessary for the library
@@ -457,9 +418,13 @@ protected:
     // SetImageList helper
     void SetAnyImageList(wxImageList *imageList, int which);
 
+    // refresh a single item
+    void RefreshItem(const wxTreeItemId& item);
+
     wxTextCtrl  *m_textCtrl;        // used while editing the item label
     wxImageList *m_imageListNormal, // images for tree elements
                 *m_imageListState;  // special images for app defined states
+    bool         m_ownsImageListNormal, m_ownsImageListState;
 
 private:
     // the common part of all ctors
@@ -486,14 +451,13 @@ private:
 
     void DeleteTextCtrl();
 
-    // support for additional item images
-    friend class wxTreeItemIndirectData;
+    // support for additional item images which we implement using
+    // wxTreeItemIndirectData technique - see the comments in msw/treectrl.cpp
     void SetIndirectItemData(const wxTreeItemId& item,
-                             wxTreeItemIndirectData *data);
+                             class wxTreeItemIndirectData *data);
     bool HasIndirectData(const wxTreeItemId& item) const;
-
-    // the array storing all item ids which have indirect data
-    wxArrayTreeItemIds m_itemsWithIndirectData;
+    bool IsDataIndirect(wxTreeItemData *data) const
+        { return data && data->GetId().m_pItem == 0; }
 
     // the hash storing the items attributes (indexed by items ids)
     wxHashTable m_attrs;
@@ -504,11 +468,19 @@ private:
     // used for dragging
     wxDragImage *m_dragImage;
 
+    // Virtual root item, if wxTR_HIDE_ROOT is set.
+    void* m_pVirtualRoot;
+
     // the starting item for selection with Shift
     WXHTREEITEM m_htSelStart;
 
+    friend class wxTreeItemIndirectData;
+    friend class wxTreeSortHelper;
+
     DECLARE_DYNAMIC_CLASS(wxTreeCtrl)
 };
 
+#endif // wxUSE_TREECTRL
+
 #endif
     // _WX_TREECTRL_H_
