@@ -1,13 +1,28 @@
+/////////////////////////////////////////////////////////////////////////////
+// Name:        wx/printdlg.h
+// Purpose:     Base header for print dialogs
+// Author:      Julian Smart
+// Modified by:
+// Created:
+// RCS-ID:      $Id: printdlg.h,v 1.10.2.2 2005/05/04 18:23:29 JS Exp $
+// Copyright:   (c) Julian Smart
+// Licence:     wxWindows Licence
+/////////////////////////////////////////////////////////////////////////////
+
 #ifndef _WX_PRINTDLG_H_BASE_
 #define _WX_PRINTDLG_H_BASE_
 
-#if defined(__WXMSW__)
+#if defined(__WXUNIVERSAL__)
+#include "wx/generic/prntdlgg.h"
+#elif defined(__WXMSW__)
 #include "wx/msw/printdlg.h"
 #elif defined(__WXMOTIF__)
 #include "wx/generic/prntdlgg.h"
 #elif defined(__WXGTK__)
 #include "wx/generic/prntdlgg.h"
-#elif defined(__WXQT__)
+#elif defined(__WXX11__)
+#include "wx/generic/prntdlgg.h"
+#elif defined(__WXMGL__)
 #include "wx/generic/prntdlgg.h"
 #elif defined(__WXMAC__)
 #include "wx/mac/printdlg.h"
@@ -17,7 +32,7 @@
 #include "wx/generic/prntdlgg.h"
 #endif
 
-#if !defined(__WXMSW__) && !defined(__WXMAC__)
+#if defined(__WXUNIVERSAL__) || (!defined(__WXMSW__) && !defined(__WXMAC__))
 #define wxPrintDialog wxGenericPrintDialog
 #define sm_classwxPrintDialog sm_classwxGenericPrintDialog
 
