@@ -4,7 +4,7 @@
 // Author:      Guillermo Rodriguez Garcia <guille@iies.es>
 // Modified by:
 // Created:     1999/09/19
-// RCS-ID:      $Id: client.cpp,v 1.17 2000/03/17 15:42:54 RR Exp $
+// RCS-ID:      $Id: client.cpp,v 1.21.2.1 2002/11/11 19:07:23 RR Exp $
 // Copyright:   (c) 1999 Guillermo Rodriguez Garcia
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -17,7 +17,7 @@
 // headers
 // --------------------------------------------------------------------------
 
-#ifdef __GNUG__
+#if defined(__GNUG__) && !defined(__APPLE__)
 #  pragma implementation "client.cpp"
 #  pragma interface "client.cpp"
 #endif
@@ -43,7 +43,7 @@
 // --------------------------------------------------------------------------
 
 // the application icon
-#if defined(__WXGTK__) || defined(__WXMOTIF__)
+#if defined(__WXGTK__) || defined(__WXX11__) || defined(__WXMOTIF__) || defined(__WXMAC__)
 #  include "mondrian.xpm"
 #endif
 
@@ -210,8 +210,7 @@ MyFrame::MyFrame() : wxFrame((wxFrame *)NULL, -1,
 
   // Make a textctrl for logging
   m_text  = new wxTextCtrl(this, -1,
-                           _("Welcome to wxSocket demo: Client\n"
-                             "Client ready\n"),
+                           _("Welcome to wxSocket demo: Client\nClient ready\n"),
                            wxDefaultPosition, wxDefaultSize,
                            wxTE_MULTILINE | wxTE_READONLY);
 
@@ -245,8 +244,7 @@ void MyFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
 
 void MyFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
 {
-  wxMessageBox(_("wxSocket demo: Client\n"
-                 "(c) 1999 Guillermo Rodriguez Garcia\n"),
+  wxMessageBox(_("wxSocket demo: Client\n(c) 1999 Guillermo Rodriguez Garcia\n"),
                _("About Client"),
                wxOK | wxICON_INFORMATION, this);
 }
@@ -334,8 +332,8 @@ void MyFrame::OnOpenConnection(wxCommandEvent& WXUNUSED(event))
 
 void MyFrame::OnTest1(wxCommandEvent& WXUNUSED(event))
 {
-  const char *buf1;
-  char       *buf2;
+  const wxChar *buf1;
+  wxChar       *buf2;
   unsigned char len;
 
   // Disable socket menu entries (exception: Close Session)
@@ -363,8 +361,8 @@ void MyFrame::OnTest1(wxCommandEvent& WXUNUSED(event))
   m_sock->SetFlags(wxSOCKET_WAITALL);
 
   buf1 = _("Test string (less than 256 chars!)");
-  len  = wxStrlen(buf1) + 1;
-  buf2 = new char[len];
+  len  = (wxStrlen(buf1) + 1) * sizeof(wxChar);
+  buf2 = new wxChar[wxStrlen(buf1) + 1];
 
   m_text->AppendText(_("Sending a test buffer to the server ..."));
   m_sock->Write(&len, 1);
@@ -395,8 +393,8 @@ void MyFrame::OnTest1(wxCommandEvent& WXUNUSED(event))
 
 void MyFrame::OnTest2(wxCommandEvent& WXUNUSED(event))
 {
-  char *msg1;
-  char *msg2;
+  const wxChar *msg1;
+  wxChar *msg2;
   size_t len;
 
   // Disable socket menu entries (exception: Close Session)
@@ -423,9 +421,9 @@ void MyFrame::OnTest2(wxCommandEvent& WXUNUSED(event))
     _("Test 2 ..."),
     _("Yes I like wxWindows!"));
 
-  msg1 = (char *)s.c_str();
-  len  = wxStrlen(msg1) + 1;
-  msg2 = new char[len];
+  msg1 = s.c_str();
+  len  = (wxStrlen(msg1) + 1) * sizeof(wxChar);
+  msg2 = new wxChar[wxStrlen(msg1) + 1];
 
   m_text->AppendText(_("Sending the string with WriteMsg ..."));
   m_sock->WriteMsg(msg1, len);
@@ -566,16 +564,14 @@ void MyFrame::OnTestURL(wxCommandEvent& WXUNUSED(event))
 
   // Print the contents type and file size
   wxString s;
-  s.Printf(_("Contents type: %s\n"
-             "File size: %i\n"
-             "Starting to download...\n"),
+  s.Printf(_("Contents type: %s\nFile size: %i\nStarting to download...\n"),
              url.GetProtocol().GetContentType().c_str(),
              data->GetSize());
   m_text->AppendText(s);
   wxYield();
 
   // Get the data
-  wxFileOutputStream sout(wxString("test.url"));
+  wxFileOutputStream sout( wxT("test.url") );
   if (!sout.Ok())
   {
     m_text->AppendText(_("Error: couldn't open file for output\n"));
