@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     01/02/97
-// RCS-ID:      $Id: event.cpp,v 1.74.2.4 2001/12/17 16:56:09 VZ Exp $
+// RCS-ID:      $Id: event.cpp,v 1.110.2.3 2003/09/15 11:00:51 VZ Exp $
 // Copyright:   (c) Julian Smart and Markus Holzem
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -52,9 +52,9 @@
 
 IMPLEMENT_DYNAMIC_CLASS(wxEvtHandler, wxObject)
 IMPLEMENT_ABSTRACT_CLASS(wxEvent, wxObject)
-IMPLEMENT_DYNAMIC_CLASS(wxIdleEvent, wxEvent)
 
 #if wxUSE_GUI
+    IMPLEMENT_DYNAMIC_CLASS(wxIdleEvent, wxEvent)
     IMPLEMENT_DYNAMIC_CLASS(wxCommandEvent, wxEvent)
     IMPLEMENT_DYNAMIC_CLASS(wxNotifyEvent, wxCommandEvent)
     IMPLEMENT_DYNAMIC_CLASS(wxScrollEvent, wxCommandEvent)
@@ -63,9 +63,11 @@ IMPLEMENT_DYNAMIC_CLASS(wxIdleEvent, wxEvent)
     IMPLEMENT_DYNAMIC_CLASS(wxKeyEvent, wxEvent)
     IMPLEMENT_DYNAMIC_CLASS(wxSizeEvent, wxEvent)
     IMPLEMENT_DYNAMIC_CLASS(wxPaintEvent, wxEvent)
+    IMPLEMENT_DYNAMIC_CLASS(wxNcPaintEvent, wxEvent)
     IMPLEMENT_DYNAMIC_CLASS(wxEraseEvent, wxEvent)
     IMPLEMENT_DYNAMIC_CLASS(wxMoveEvent, wxEvent)
     IMPLEMENT_DYNAMIC_CLASS(wxFocusEvent, wxEvent)
+    IMPLEMENT_DYNAMIC_CLASS(wxChildFocusEvent, wxCommandEvent)
     IMPLEMENT_DYNAMIC_CLASS(wxCloseEvent, wxEvent)
     IMPLEMENT_DYNAMIC_CLASS(wxShowEvent, wxEvent)
     IMPLEMENT_DYNAMIC_CLASS(wxMaximizeEvent, wxEvent)
@@ -75,13 +77,18 @@ IMPLEMENT_DYNAMIC_CLASS(wxIdleEvent, wxEvent)
     IMPLEMENT_DYNAMIC_CLASS(wxDropFilesEvent, wxEvent)
     IMPLEMENT_DYNAMIC_CLASS(wxActivateEvent, wxEvent)
     IMPLEMENT_DYNAMIC_CLASS(wxInitDialogEvent, wxEvent)
+    IMPLEMENT_DYNAMIC_CLASS(wxSetCursorEvent, wxEvent)
     IMPLEMENT_DYNAMIC_CLASS(wxSysColourChangedEvent, wxEvent)
+    IMPLEMENT_DYNAMIC_CLASS(wxDisplayChangedEvent, wxEvent)
     IMPLEMENT_DYNAMIC_CLASS(wxUpdateUIEvent, wxCommandEvent)
-    IMPLEMENT_DYNAMIC_CLASS(wxNavigationKeyEvent, wxCommandEvent)
+    IMPLEMENT_DYNAMIC_CLASS(wxNavigationKeyEvent, wxEvent)
     IMPLEMENT_DYNAMIC_CLASS(wxPaletteChangedEvent, wxEvent)
     IMPLEMENT_DYNAMIC_CLASS(wxQueryNewPaletteEvent, wxEvent)
     IMPLEMENT_DYNAMIC_CLASS(wxWindowCreateEvent, wxEvent)
     IMPLEMENT_DYNAMIC_CLASS(wxWindowDestroyEvent, wxEvent)
+    IMPLEMENT_DYNAMIC_CLASS(wxHelpEvent, wxCommandEvent)
+    IMPLEMENT_DYNAMIC_CLASS(wxContextMenuEvent, wxCommandEvent)
+    IMPLEMENT_DYNAMIC_CLASS(wxMouseCaptureChangedEvent, wxEvent)
 #endif // wxUSE_GUI
 
 const wxEventTable *wxEvtHandler::GetEventTable() const
@@ -91,8 +98,7 @@ const wxEventTable wxEvtHandler::sm_eventTable =
     { (const wxEventTable *)NULL, &wxEvtHandler::sm_eventTableEntries[0] };
 
 const wxEventTableEntry wxEvtHandler::sm_eventTableEntries[] =
-    { { 0, 0, 0, (wxObjectEventFunction) NULL, (wxObject*) NULL } };
-
+    { DECLARE_EVENT_TABLE_ENTRY(wxEVT_NULL, 0, 0, (wxObjectEventFunction)NULL, NULL) };
 
 // ----------------------------------------------------------------------------
 // global variables
@@ -106,10 +112,178 @@ wxList *wxPendingEvents = (wxList *)NULL;
     wxCriticalSection *wxPendingEventsLocker = (wxCriticalSection *)NULL;
 #endif
 
+#if !WXWIN_COMPATIBILITY_EVENT_TYPES
+
+// common event types are defined here, other event types are defined by the
+// components which use them
+
+DEFINE_EVENT_TYPE(wxEVT_NULL)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_BUTTON_CLICKED)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_CHECKBOX_CLICKED)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_CHOICE_SELECTED)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_LISTBOX_SELECTED)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_LISTBOX_DOUBLECLICKED)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_CHECKLISTBOX_TOGGLED)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_MENU_SELECTED)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_SLIDER_UPDATED)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_RADIOBOX_SELECTED)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_RADIOBUTTON_SELECTED)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_SCROLLBAR_UPDATED)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_VLBOX_SELECTED)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_COMBOBOX_SELECTED)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_TOOL_RCLICKED)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_TOOL_ENTER)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_SPINCTRL_UPDATED)
+
+// Sockets and timers send events, too
+DEFINE_EVENT_TYPE(wxEVT_SOCKET)
+DEFINE_EVENT_TYPE(wxEVT_TIMER)
+
+// Mouse event types
+DEFINE_EVENT_TYPE(wxEVT_LEFT_DOWN)
+DEFINE_EVENT_TYPE(wxEVT_LEFT_UP)
+DEFINE_EVENT_TYPE(wxEVT_MIDDLE_DOWN)
+DEFINE_EVENT_TYPE(wxEVT_MIDDLE_UP)
+DEFINE_EVENT_TYPE(wxEVT_RIGHT_DOWN)
+DEFINE_EVENT_TYPE(wxEVT_RIGHT_UP)
+DEFINE_EVENT_TYPE(wxEVT_MOTION)
+DEFINE_EVENT_TYPE(wxEVT_ENTER_WINDOW)
+DEFINE_EVENT_TYPE(wxEVT_LEAVE_WINDOW)
+DEFINE_EVENT_TYPE(wxEVT_LEFT_DCLICK)
+DEFINE_EVENT_TYPE(wxEVT_MIDDLE_DCLICK)
+DEFINE_EVENT_TYPE(wxEVT_RIGHT_DCLICK)
+DEFINE_EVENT_TYPE(wxEVT_SET_FOCUS)
+DEFINE_EVENT_TYPE(wxEVT_KILL_FOCUS)
+DEFINE_EVENT_TYPE(wxEVT_CHILD_FOCUS)
+DEFINE_EVENT_TYPE(wxEVT_MOUSEWHEEL)
+
+// Non-client mouse events
+DEFINE_EVENT_TYPE(wxEVT_NC_LEFT_DOWN)
+DEFINE_EVENT_TYPE(wxEVT_NC_LEFT_UP)
+DEFINE_EVENT_TYPE(wxEVT_NC_MIDDLE_DOWN)
+DEFINE_EVENT_TYPE(wxEVT_NC_MIDDLE_UP)
+DEFINE_EVENT_TYPE(wxEVT_NC_RIGHT_DOWN)
+DEFINE_EVENT_TYPE(wxEVT_NC_RIGHT_UP)
+DEFINE_EVENT_TYPE(wxEVT_NC_MOTION)
+DEFINE_EVENT_TYPE(wxEVT_NC_ENTER_WINDOW)
+DEFINE_EVENT_TYPE(wxEVT_NC_LEAVE_WINDOW)
+DEFINE_EVENT_TYPE(wxEVT_NC_LEFT_DCLICK)
+DEFINE_EVENT_TYPE(wxEVT_NC_MIDDLE_DCLICK)
+DEFINE_EVENT_TYPE(wxEVT_NC_RIGHT_DCLICK)
+
+// Character input event type
+DEFINE_EVENT_TYPE(wxEVT_CHAR)
+DEFINE_EVENT_TYPE(wxEVT_CHAR_HOOK)
+DEFINE_EVENT_TYPE(wxEVT_NAVIGATION_KEY)
+DEFINE_EVENT_TYPE(wxEVT_KEY_DOWN)
+DEFINE_EVENT_TYPE(wxEVT_KEY_UP)
+
+// Set cursor event
+DEFINE_EVENT_TYPE(wxEVT_SET_CURSOR)
+
+// wxScrollbar and wxSlider event identifiers
+DEFINE_EVENT_TYPE(wxEVT_SCROLL_TOP)
+DEFINE_EVENT_TYPE(wxEVT_SCROLL_BOTTOM)
+DEFINE_EVENT_TYPE(wxEVT_SCROLL_LINEUP)
+DEFINE_EVENT_TYPE(wxEVT_SCROLL_LINEDOWN)
+DEFINE_EVENT_TYPE(wxEVT_SCROLL_PAGEUP)
+DEFINE_EVENT_TYPE(wxEVT_SCROLL_PAGEDOWN)
+DEFINE_EVENT_TYPE(wxEVT_SCROLL_THUMBTRACK)
+DEFINE_EVENT_TYPE(wxEVT_SCROLL_THUMBRELEASE)
+DEFINE_EVENT_TYPE(wxEVT_SCROLL_ENDSCROLL)
+
+// Scroll events from wxWindow
+DEFINE_EVENT_TYPE(wxEVT_SCROLLWIN_TOP)
+DEFINE_EVENT_TYPE(wxEVT_SCROLLWIN_BOTTOM)
+DEFINE_EVENT_TYPE(wxEVT_SCROLLWIN_LINEUP)
+DEFINE_EVENT_TYPE(wxEVT_SCROLLWIN_LINEDOWN)
+DEFINE_EVENT_TYPE(wxEVT_SCROLLWIN_PAGEUP)
+DEFINE_EVENT_TYPE(wxEVT_SCROLLWIN_PAGEDOWN)
+DEFINE_EVENT_TYPE(wxEVT_SCROLLWIN_THUMBTRACK)
+DEFINE_EVENT_TYPE(wxEVT_SCROLLWIN_THUMBRELEASE)
+
+// System events
+DEFINE_EVENT_TYPE(wxEVT_SIZE)
+DEFINE_EVENT_TYPE(wxEVT_MOVE)
+DEFINE_EVENT_TYPE(wxEVT_CLOSE_WINDOW)
+DEFINE_EVENT_TYPE(wxEVT_END_SESSION)
+DEFINE_EVENT_TYPE(wxEVT_QUERY_END_SESSION)
+DEFINE_EVENT_TYPE(wxEVT_ACTIVATE_APP)
+DEFINE_EVENT_TYPE(wxEVT_POWER)
+DEFINE_EVENT_TYPE(wxEVT_ACTIVATE)
+DEFINE_EVENT_TYPE(wxEVT_CREATE)
+DEFINE_EVENT_TYPE(wxEVT_DESTROY)
+DEFINE_EVENT_TYPE(wxEVT_SHOW)
+DEFINE_EVENT_TYPE(wxEVT_ICONIZE)
+DEFINE_EVENT_TYPE(wxEVT_MAXIMIZE)
+DEFINE_EVENT_TYPE(wxEVT_MOUSE_CAPTURE_CHANGED)
+DEFINE_EVENT_TYPE(wxEVT_PAINT)
+DEFINE_EVENT_TYPE(wxEVT_ERASE_BACKGROUND)
+DEFINE_EVENT_TYPE(wxEVT_NC_PAINT)
+DEFINE_EVENT_TYPE(wxEVT_PAINT_ICON)
+DEFINE_EVENT_TYPE(wxEVT_MENU_OPEN)
+DEFINE_EVENT_TYPE(wxEVT_MENU_CLOSE)
+DEFINE_EVENT_TYPE(wxEVT_MENU_HIGHLIGHT)
+DEFINE_EVENT_TYPE(wxEVT_CONTEXT_MENU)
+DEFINE_EVENT_TYPE(wxEVT_SYS_COLOUR_CHANGED)
+DEFINE_EVENT_TYPE(wxEVT_DISPLAY_CHANGED)
+DEFINE_EVENT_TYPE(wxEVT_SETTING_CHANGED)
+DEFINE_EVENT_TYPE(wxEVT_QUERY_NEW_PALETTE)
+DEFINE_EVENT_TYPE(wxEVT_PALETTE_CHANGED)
+DEFINE_EVENT_TYPE(wxEVT_JOY_BUTTON_DOWN)
+DEFINE_EVENT_TYPE(wxEVT_JOY_BUTTON_UP)
+DEFINE_EVENT_TYPE(wxEVT_JOY_MOVE)
+DEFINE_EVENT_TYPE(wxEVT_JOY_ZMOVE)
+DEFINE_EVENT_TYPE(wxEVT_DROP_FILES)
+DEFINE_EVENT_TYPE(wxEVT_DRAW_ITEM)
+DEFINE_EVENT_TYPE(wxEVT_MEASURE_ITEM)
+DEFINE_EVENT_TYPE(wxEVT_COMPARE_ITEM)
+DEFINE_EVENT_TYPE(wxEVT_INIT_DIALOG)
+DEFINE_EVENT_TYPE(wxEVT_IDLE)
+DEFINE_EVENT_TYPE(wxEVT_UPDATE_UI)
+
+// Generic command events
+// Note: a click is a higher-level event than button down/up
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_LEFT_CLICK)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_LEFT_DCLICK)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_RIGHT_CLICK)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_RIGHT_DCLICK)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_SET_FOCUS)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_KILL_FOCUS)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_ENTER)
+
+// Help events
+DEFINE_EVENT_TYPE(wxEVT_HELP)
+DEFINE_EVENT_TYPE(wxEVT_DETAILED_HELP)
+
+#endif // !WXWIN_COMPATIBILITY_EVENT_TYPES
+
 // ============================================================================
 // implementation
 // ============================================================================
 
+// ----------------------------------------------------------------------------
+// event initialization
+// ----------------------------------------------------------------------------
+
+int wxNewEventType()
+{
+    // MT-FIXME
+    static int s_lastUsedEventType = wxEVT_FIRST;
+
+#if WXWIN_COMPATIBILITY_2
+    // check that we don't overlap with the user-defined types: if it does
+    // happen, the best solution is probably to update the existing code to
+    // use wxNewEventType() instead of wxEVT_USER_FIRST
+    //
+    // due to the uncertainty
+    wxASSERT_MSG( s_lastUsedEventType < wxEVT_USER_FIRST - 1,
+                  _T("possible event type conflict") );
+#endif // WXWIN_COMPATIBILITY_2
+
+    return s_lastUsedEventType++;
+}
+
 // ----------------------------------------------------------------------------
 // wxEvent
 // ----------------------------------------------------------------------------
@@ -123,9 +297,9 @@ wxList *wxPendingEvents = (wxList *)NULL;
  *
  */
 
-wxEvent::wxEvent(int theId)
+wxEvent::wxEvent(int theId, wxEventType commandType )
 {
-    m_eventType = wxEVT_NULL;
+    m_eventType = commandType;
     m_eventObject = (wxObject *) NULL;
     m_timeStamp = 0;
     m_id = theId;
@@ -134,18 +308,16 @@ wxEvent::wxEvent(int theId)
     m_isCommandEvent = FALSE;
 }
 
-void wxEvent::CopyObject(wxObject& object_dest) const
+wxEvent::wxEvent(const wxEvent &src)
+    : wxObject()
+    , m_eventObject(src.m_eventObject)
+    , m_eventType(src.m_eventType)
+    , m_timeStamp(src.m_timeStamp)
+    , m_id(src.m_id)
+    , m_callbackUserData(src.m_callbackUserData)
+    , m_skipped(src.m_skipped)
+    , m_isCommandEvent(src.m_isCommandEvent)
 {
-    wxEvent *obj = (wxEvent *)&object_dest;
-    wxObject::CopyObject(object_dest);
-
-    obj->m_eventType = m_eventType;
-    obj->m_eventObject = m_eventObject;
-    obj->m_timeStamp = m_timeStamp;
-    obj->m_id = m_id;
-    obj->m_skipped = m_skipped;
-    obj->m_callbackUserData = m_callbackUserData;
-    obj->m_isCommandEvent = m_isCommandEvent;
 }
 
 #if wxUSE_GUI
@@ -156,43 +328,16 @@ void wxEvent::CopyObject(wxObject& object_dest) const
  */
 
 wxCommandEvent::wxCommandEvent(wxEventType commandType, int theId)
+  : wxEvent(theId, commandType)
 {
-    m_eventType = commandType;
     m_clientData = (char *) NULL;
     m_clientObject = (wxClientData *) NULL;
     m_extraLong = 0;
     m_commandInt = 0;
-    m_id = theId;
     m_commandString = wxEmptyString;
     m_isCommandEvent = TRUE;
 }
 
-void wxCommandEvent::CopyObject(wxObject& obj_d) const
-{
-    wxCommandEvent *obj = (wxCommandEvent *)&obj_d;
-
-    wxEvent::CopyObject(obj_d);
-
-    obj->m_clientData    = m_clientData;
-    obj->m_clientObject  = m_clientObject;
-    obj->m_extraLong     = m_extraLong;
-    obj->m_commandInt    = m_commandInt;
-    obj->m_commandString = m_commandString;
-}
-
-/*
- * Notify events
- */
-
-void wxNotifyEvent::CopyObject(wxObject& obj_d) const
-{
-    wxNotifyEvent *obj = (wxNotifyEvent *)&obj_d;
-
-    wxEvent::CopyObject(obj_d);
-
-    if (!m_bAllow) obj->Veto();
-}
-
 /*
  * Scroll events
  */
@@ -201,7 +346,7 @@ wxScrollEvent::wxScrollEvent(wxEventType commandType,
                              int id,
                              int pos,
                              int orient)
-             : wxCommandEvent(commandType, id)
+    : wxCommandEvent(commandType, id)
 {
     m_extraLong = orient;
     m_commandInt = pos;
@@ -220,16 +365,6 @@ wxScrollWinEvent::wxScrollWinEvent(wxEventType commandType,
     m_commandInt = pos;
 }
 
-void wxScrollWinEvent::CopyObject(wxObject& obj_d) const
-{
-    wxScrollWinEvent *obj = (wxScrollWinEvent*)&obj_d;
-
-    wxEvent::CopyObject(obj_d);
-
-    obj->m_extraLong    = m_extraLong;
-    obj->m_commandInt   = m_commandInt;
-}
-
 /*
  * Mouse events
  *
@@ -247,23 +382,30 @@ wxMouseEvent::wxMouseEvent(wxEventType commandType)
     m_middleDown = FALSE;
     m_x = 0;
     m_y = 0;
+    m_wheelRotation = 0;
+    m_wheelDelta = 0;
+    m_linesPerAction = 0;
 }
 
-void wxMouseEvent::CopyObject(wxObject& obj_d) const
+void wxMouseEvent::Assign(const wxMouseEvent& event)
 {
-    wxMouseEvent *obj = (wxMouseEvent *)&obj_d;
+    m_eventType = event.m_eventType;
 
-    wxEvent::CopyObject(obj_d);
+    m_x = event.m_x;
+    m_y = event.m_y;
 
-    obj->m_metaDown = m_metaDown;
-    obj->m_altDown = m_altDown;
-    obj->m_controlDown = m_controlDown;
-    obj->m_shiftDown = m_shiftDown;
-    obj->m_leftDown = m_leftDown;
-    obj->m_rightDown = m_rightDown;
-    obj->m_middleDown = m_middleDown;
-    obj->m_x = m_x;
-    obj->m_y = m_y;
+    m_leftDown = event.m_leftDown;
+    m_middleDown = event.m_middleDown;
+    m_rightDown = event.m_rightDown;
+
+    m_controlDown = event.m_controlDown;
+    m_shiftDown = event.m_shiftDown;
+    m_altDown = event.m_altDown;
+    m_metaDown = event.m_metaDown;
+
+    m_wheelRotation = event.m_wheelRotation;
+    m_wheelDelta = event.m_wheelDelta;
+    m_linesPerAction = event.m_linesPerAction;
 }
 
 // True if was a button dclick event (1 = left, 2 = middle, 3 = right)
@@ -312,7 +454,8 @@ bool wxMouseEvent::ButtonDown(int but) const
 // or any button up event (but = -1)
 bool wxMouseEvent::ButtonUp(int but) const
 {
-    switch (but) {
+    switch (but)
+    {
         case -1:
             return (LeftUp() || MiddleUp() || RightUp());
         case 1:
@@ -331,7 +474,8 @@ bool wxMouseEvent::ButtonUp(int but) const
 // True if the given button is currently changing state
 bool wxMouseEvent::Button(int but) const
 {
-    switch (but) {
+    switch (but)
+    {
         case -1:
             return (ButtonUp(-1) || ButtonDown(-1) || ButtonDClick(-1));
         case 1:
@@ -349,7 +493,8 @@ bool wxMouseEvent::Button(int but) const
 
 bool wxMouseEvent::ButtonIsDown(int but) const
 {
-    switch (but) {
+    switch (but)
+    {
         case -1:
             return (LeftIsDown() || MiddleIsDown() || RightIsDown());
         case 1:
@@ -365,6 +510,19 @@ bool wxMouseEvent::ButtonIsDown(int but) const
     return FALSE;
 }
 
+int wxMouseEvent::GetButton() const
+{
+    for ( int i = 1; i <= 3; i++ )
+    {
+        if ( Button(i) )
+        {
+            return i;
+        }
+    }
+
+    return -1;
+}
+
 // Find the logical position of the event given the DC
 wxPoint wxMouseEvent::GetLogicalPosition(const wxDC& dc) const
 {
@@ -374,7 +532,7 @@ wxPoint wxMouseEvent::GetLogicalPosition(const wxDC& dc) const
 
 
 /*
- * Keyboard events
+ * Keyboard event
  *
  */
 
@@ -387,140 +545,30 @@ wxKeyEvent::wxKeyEvent(wxEventType type)
     m_altDown = FALSE;
     m_keyCode = 0;
     m_scanCode = 0;
-}
-
-void wxKeyEvent::CopyObject(wxObject& obj_d) const
-{
-    wxKeyEvent *obj = (wxKeyEvent *)&obj_d;
-    wxEvent::CopyObject(obj_d);
-
-    obj->m_x = m_x;
-    obj->m_y = m_y;
-    obj->m_keyCode = m_keyCode;
-
-    obj->m_shiftDown   = m_shiftDown;
-    obj->m_controlDown = m_controlDown;
-    obj->m_metaDown    = m_metaDown;
-    obj->m_altDown     = m_altDown;
-    obj->m_keyCode     = m_keyCode;
-}
-
-
-/*
- * Misc events
- */
-
-void wxSizeEvent::CopyObject(wxObject& obj_d) const
-{
-    wxSizeEvent *obj = (wxSizeEvent *)&obj_d;
-    wxEvent::CopyObject(obj_d);
-
-    obj->m_size = m_size;
-}
-
-void wxMoveEvent::CopyObject(wxObject& obj_d) const
-{
-    wxMoveEvent *obj = (wxMoveEvent *)&obj_d;
-    wxEvent::CopyObject(obj_d);
-
-    obj->m_pos = m_pos;
-}
-
-void wxEraseEvent::CopyObject(wxObject& obj_d) const
-{
-    wxEraseEvent *obj = (wxEraseEvent *)&obj_d;
-    wxEvent::CopyObject(obj_d);
-
-    obj->m_dc = m_dc;
-}
-
-void wxActivateEvent::CopyObject(wxObject& obj_d) const
-{
-    wxActivateEvent *obj = (wxActivateEvent *)&obj_d;
-    wxEvent::CopyObject(obj_d);
-
-    obj->m_active = m_active;
-}
-
-void wxMenuEvent::CopyObject(wxObject& obj_d) const
-{
-    wxMenuEvent *obj = (wxMenuEvent *)&obj_d;
-    wxEvent::CopyObject(obj_d);
-
-    obj->m_menuId = m_menuId;
-}
-
-void wxCloseEvent::CopyObject(wxObject& obj_d) const
-{
-    wxCloseEvent *obj = (wxCloseEvent *)&obj_d;
-    wxEvent::CopyObject(obj_d);
-
-    obj->m_loggingOff = m_loggingOff;
-    obj->m_veto = m_veto;
-#if WXWIN_COMPATIBILITY
-    obj->m_force = m_force;
+#if wxUSE_UNICODE
+    m_uniChar = 0;
 #endif
-    obj->m_canVeto = m_canVeto;
 }
 
-void wxShowEvent::CopyObject(wxObject& obj_d) const
+wxKeyEvent::wxKeyEvent(const wxKeyEvent& evt)
+    : wxEvent(evt)
 {
-    wxShowEvent *obj = (wxShowEvent *)&obj_d;
-    wxEvent::CopyObject(obj_d);
+    m_x = evt.m_x;
+    m_y = evt.m_y;
 
-    obj->m_show = m_show;
-}
+    m_keyCode = evt.m_keyCode;
 
-void wxJoystickEvent::CopyObject(wxObject& obj_d) const
-{
-    wxJoystickEvent *obj = (wxJoystickEvent *)&obj_d;
-    wxEvent::CopyObject(obj_d);
+    m_controlDown = evt.m_controlDown;
+    m_shiftDown = evt.m_shiftDown;
+    m_altDown = evt.m_altDown;
+    m_metaDown = evt.m_metaDown;
+    m_scanCode = evt.m_scanCode;
+    m_rawCode = evt.m_rawCode;
+    m_rawFlags = evt.m_rawFlags;
 
-    obj->m_pos = m_pos;
-    obj->m_zPosition = m_zPosition;
-    obj->m_buttonChange = m_buttonChange;
-    obj->m_buttonState = m_buttonState;
-    obj->m_joyStick = m_joyStick;
-}
-
-void wxDropFilesEvent::CopyObject(wxObject& obj_d) const
-{
-    wxDropFilesEvent *obj = (wxDropFilesEvent *)&obj_d;
-    wxEvent::CopyObject(obj_d);
-
-    obj->m_noFiles = m_noFiles;
-    obj->m_pos = m_pos;
-    // TODO: Problem with obj->m_files. It should be deallocated by the
-    // destructor of the event.
-}
-
-void wxUpdateUIEvent::CopyObject(wxObject &obj_d) const
-{
-    wxUpdateUIEvent *obj = (wxUpdateUIEvent *)&obj_d;
-    wxEvent::CopyObject(obj_d);
-
-    obj->m_checked = m_checked;
-    obj->m_enabled = m_enabled;
-    obj->m_text = m_text;
-    obj->m_setText = m_setText;
-    obj->m_setChecked = m_setChecked;
-    obj->m_setEnabled = m_setEnabled;
-}
-
-void wxPaletteChangedEvent::CopyObject(wxObject &obj_d) const
-{
-    wxPaletteChangedEvent *obj = (wxPaletteChangedEvent *)&obj_d;
-    wxEvent::CopyObject(obj_d);
-
-    obj->m_changedWindow = m_changedWindow;
-}
-
-void wxQueryNewPaletteEvent::CopyObject(wxObject& obj_d) const
-{
-    wxQueryNewPaletteEvent *obj = (wxQueryNewPaletteEvent *)&obj_d;
-    wxEvent::CopyObject(obj_d);
-
-    obj->m_paletteRealized = m_paletteRealized;
+#if wxUSE_UNICODE
+    m_uniChar = evt.m_uniChar;
+#endif
 }
 
 wxWindowCreateEvent::wxWindowCreateEvent(wxWindow *win)
@@ -535,15 +583,17 @@ wxWindowDestroyEvent::wxWindowDestroyEvent(wxWindow *win)
     SetEventObject(win);
 }
 
+wxChildFocusEvent::wxChildFocusEvent(wxWindow *win)
+                 : wxCommandEvent(wxEVT_CHILD_FOCUS)
+{
+    SetEventObject(win);
+}
+
 #endif // wxUSE_GUI
 
-void wxIdleEvent::CopyObject(wxObject& obj_d) const
-{
-    wxIdleEvent *obj = (wxIdleEvent *)&obj_d;
-    wxEvent::CopyObject(obj_d);
-
-    obj->m_requestMore = m_requestMore;
-}
+// ----------------------------------------------------------------------------
+// wxEvtHandler
+// ----------------------------------------------------------------------------
 
 /*
  * Event handler
@@ -562,6 +612,9 @@ wxEvtHandler::wxEvtHandler()
     m_eventsLocker = new wxCriticalSection;
 #  endif
 #endif
+    // no client data (yet)
+    m_clientData = NULL;
+    m_clientDataType = wxClientData_None;
 }
 
 wxEvtHandler::~wxEvtHandler()
@@ -578,8 +631,14 @@ wxEvtHandler::~wxEvtHandler()
         wxNode *node = m_dynamicEvents->First();
         while (node)
         {
+#if WXWIN_COMPATIBILITY_EVENT_TYPES
             wxEventTableEntry *entry = (wxEventTableEntry*)node->Data();
-            if (entry->m_callbackUserData) delete entry->m_callbackUserData;
+#else // !WXWIN_COMPATIBILITY_EVENT_TYPES
+            wxDynamicEventTableEntry *entry = (wxDynamicEventTableEntry*)node->Data();
+#endif // WXWIN_COMPATIBILITY_EVENT_TYPES/!WXWIN_COMPATIBILITY_EVENT_TYPES
+
+            if (entry->m_callbackUserData)
+                delete entry->m_callbackUserData;
             delete entry;
             node = node->Next();
         }
@@ -593,6 +652,10 @@ wxEvtHandler::~wxEvtHandler()
     delete m_eventsLocker;
 #  endif
 #endif
+
+    // we only delete object data, not untyped
+    if ( m_clientDataType == wxClientData_Object )
+        delete m_clientObject;
 }
 
 #if wxUSE_THREADS
@@ -614,6 +677,13 @@ void wxEvtHandler::AddPendingEvent(wxEvent& event)
 {
     // 1) Add event to list of pending events of this event handler
 
+    wxEvent *eventCopy = event.Clone();
+
+    // we must be able to copy the events here so the event class must
+    // implement Clone() properly instead of just providing a NULL stab for it
+    wxCHECK_RET( eventCopy,
+                 _T("events of this type aren't supposed to be posted") );
+
 #if defined(__VISAGECPP__)
     wxENTER_CRIT_SECT( m_eventsLocker);
 #else
@@ -623,9 +693,7 @@ void wxEvtHandler::AddPendingEvent(wxEvent& event)
     if ( !m_pendingEvents )
       m_pendingEvents = new wxList;
 
-    wxEvent *event2 = (wxEvent *)event.Clone();
-
-    m_pendingEvents->Append(event2);
+    m_pendingEvents->Append(eventCopy);
 
 #if defined(__VISAGECPP__)
     wxLEAVE_CRIT_SECT( m_eventsLocker);
@@ -664,7 +732,7 @@ void wxEvtHandler::ProcessPendingEvents()
         delete node;
 
         // In ProcessEvent, new events might get added and
-	    // we can safely leave the crtical section here.
+        // we can safely leave the crtical section here.
 #if defined(__VISAGECPP__)
         wxLEAVE_CRIT_SECT( m_eventsLocker);
 #else
@@ -695,10 +763,58 @@ void wxEvtHandler::ProcessPendingEvents()
 bool wxEvtHandler::ProcessEvent(wxEvent& event)
 {
 #if wxUSE_GUI
+
+    // We have to use the actual window or processing events from
+    // wxWindowNative destructor won't work (we don't see the wxWindow class)
+#ifdef __WXDEBUG__
     // check that our flag corresponds to reality
-    wxASSERT( m_isWindow == IsKindOf(CLASSINFO(wxWindow)) );
+    wxClassInfo* info = NULL;
+#ifdef __WXUNIVERSAL__
+#  if defined(__WXMSW__)
+    info = CLASSINFO(wxWindowMSW);
+#  elif defined(__WXGTK__)
+    info = CLASSINFO(wxWindowGTK);
+#  elif defined(__WXX11__)
+    info = CLASSINFO(wxWindowX11);
+#  elif defined(__WXMGL__)
+    info = CLASSINFO(wxWindowMGL);
+#  elif defined(__WXPM__)
+    info = CLASSINFO(wxWindowOS2);
+#  elif defined(__WXMAC__)
+    info = CLASSINFO(wxWindowMac);
+#  elif defined(__WXMOTIF__)
+    info = CLASSINFO(wxWindowMotif);
+#  endif
+#else
+    info = CLASSINFO(wxWindow);
+#endif
+
+    if ( m_isWindow != IsKindOf(info) )
+    {
+        wxString msg = GetClassInfo()->GetClassName();
+        msg += _T(" should [not] be a window but it is [not]");
+
+        wxFAIL_MSG( msg );
+    }
+
+#endif // __WXDEBUG__
+
 #endif // wxUSE_GUI
 
+    // allow the application to hook into event processing
+    if ( wxTheApp )
+    {
+        int rc = wxTheApp->FilterEvent(event);
+        if ( rc != -1 )
+        {
+            wxASSERT_MSG( rc == 1 || rc == 0,
+                          _T("unexpected wxApp::FilterEvent return value") );
+
+            return rc != 0;
+        }
+        //else: proceed normally
+    }
+
     // An event handler can be enabled or disabled
     if ( GetEvtHandlerEnabled() )
     {
@@ -772,9 +888,16 @@ bool wxEvtHandler::ProcessEvent(wxEvent& event)
     {
         wxWindow *win = (wxWindow *)this;
 
-        wxWindow *parent = win->GetParent();
-        if (parent && !parent->IsBeingDeleted())
-            return parent->GetEventHandler()->ProcessEvent(event);
+        // honour the requests to stop propagation at this window: this is
+        // used by the dialogs, for example, to prevent processing the events
+        // from the dialog controls in the parent frame which rarely, if ever,
+        // makes sense
+        if ( !(win->GetExtraStyle() & wxWS_EX_BLOCK_EVENTS) )
+        {
+            wxWindow *parent = win->GetParent();
+            if ( parent && !parent->IsBeingDeleted() )
+                return parent->GetEventHandler()->ProcessEvent(event);
+        }
     }
 #endif // wxUSE_GUI
 
@@ -839,21 +962,27 @@ bool wxEvtHandler::SearchEventTable(wxEventTable& table, wxEvent& event)
 }
 
 void wxEvtHandler::Connect( int id, int lastId,
-                            wxEventType eventType,
+                            int eventType,
                             wxObjectEventFunction func,
                             wxObject *userData )
 {
+#if WXWIN_COMPATIBILITY_EVENT_TYPES
     wxEventTableEntry *entry = new wxEventTableEntry;
+    entry->m_eventType = eventType;
     entry->m_id = id;
     entry->m_lastId = lastId;
-    entry->m_eventType = eventType;
     entry->m_fn = func;
     entry->m_callbackUserData = userData;
+#else // !WXWIN_COMPATIBILITY_EVENT_TYPES
+    wxDynamicEventTableEntry *entry =
+        new wxDynamicEventTableEntry(eventType, id, lastId, func, userData);
+#endif // WXWIN_COMPATIBILITY_EVENT_TYPES/!WXWIN_COMPATIBILITY_EVENT_TYPES
 
     if (!m_dynamicEvents)
         m_dynamicEvents = new wxList;
 
-    m_dynamicEvents->Append( (wxObject*) entry );
+    // Insert at the front of the list so most recent additions are found first
+    m_dynamicEvents->Insert( (wxObject*) entry );
 }
 
 bool wxEvtHandler::Disconnect( int id, int lastId, wxEventType eventType,
@@ -866,14 +995,20 @@ bool wxEvtHandler::Disconnect( int id, int lastId, wxEventType eventType,
     wxNode *node = m_dynamicEvents->First();
     while (node)
     {
-        wxEventTableEntry *entry = (wxEventTableEntry*)node->Data();
+#if WXWIN_COMPATIBILITY_EVENT_TYPES
+            wxEventTableEntry *entry = (wxEventTableEntry*)node->Data();
+#else // !WXWIN_COMPATIBILITY_EVENT_TYPES
+            wxDynamicEventTableEntry *entry = (wxDynamicEventTableEntry*)node->Data();
+#endif // WXWIN_COMPATIBILITY_EVENT_TYPES/!WXWIN_COMPATIBILITY_EVENT_TYPES
+
         if ((entry->m_id == id) &&
             ((entry->m_lastId == lastId) || (lastId == -1)) &&
             ((entry->m_eventType == eventType) || (eventType == wxEVT_NULL)) &&
             ((entry->m_fn == func) || (func == (wxObjectEventFunction)NULL)) &&
             ((entry->m_callbackUserData == userData) || (userData == (wxObject*)NULL)))
         {
-            if (entry->m_callbackUserData) delete entry->m_callbackUserData;
+            if (entry->m_callbackUserData)
+                delete entry->m_callbackUserData;
             m_dynamicEvents->DeleteNode( node );
             delete entry;
             return TRUE;
@@ -893,7 +1028,11 @@ bool wxEvtHandler::SearchDynamicEventTable( wxEvent& event )
     wxNode *node = m_dynamicEvents->First();
     while (node)
     {
-        wxEventTableEntry *entry = (wxEventTableEntry*)node->Data();
+#if WXWIN_COMPATIBILITY_EVENT_TYPES
+            wxEventTableEntry *entry = (wxEventTableEntry*)node->Data();
+#else // !WXWIN_COMPATIBILITY_EVENT_TYPES
+            wxDynamicEventTableEntry *entry = (wxDynamicEventTableEntry*)node->Data();
+#endif // WXWIN_COMPATIBILITY_EVENT_TYPES/!WXWIN_COMPATIBILITY_EVENT_TYPES
 
         if (entry->m_fn)
         {
@@ -909,9 +1048,7 @@ bool wxEvtHandler::SearchDynamicEventTable( wxEvent& event )
 
                 (this->*((wxEventFunction) (entry->m_fn)))(event);
 
-                if (event.GetSkipped())
-                    return FALSE;
-                else
+                if ( ! event.GetSkipped() )
                     return TRUE;
             }
         }
@@ -920,6 +1057,48 @@ bool wxEvtHandler::SearchDynamicEventTable( wxEvent& event )
     return FALSE;
 };
 
+void wxEvtHandler::DoSetClientObject( wxClientData *data )
+{
+    wxASSERT_MSG( m_clientDataType != wxClientData_Void,
+                  wxT("can't have both object and void client data") );
+
+    if ( m_clientObject )
+        delete m_clientObject;
+
+    m_clientObject = data;
+    m_clientDataType = wxClientData_Object;
+}
+
+wxClientData *wxEvtHandler::DoGetClientObject() const
+{
+    // it's not an error to call GetClientObject() on a window which doesn't
+    // have client data at all - NULL will be returned
+    wxASSERT_MSG( m_clientDataType != wxClientData_Void,
+                  wxT("this window doesn't have object client data") );
+
+    return m_clientObject;
+}
+
+void wxEvtHandler::DoSetClientData( void *data )
+{
+    wxASSERT_MSG( m_clientDataType != wxClientData_Object,
+                  wxT("can't have both object and void client data") );
+
+    m_clientData = data;
+    m_clientDataType = wxClientData_Void;
+}
+
+void *wxEvtHandler::DoGetClientData() const
+{
+    // it's not an error to call GetClientData() on a window which doesn't have
+    // client data at all - NULL will be returned
+    wxASSERT_MSG( m_clientDataType != wxClientData_Object,
+                  wxT("this window doesn't have void client data") );
+
+    return m_clientData;
+}
+
+
 #if WXWIN_COMPATIBILITY
 bool wxEvtHandler::OnClose()
 {
