@@ -2,7 +2,7 @@
 // Name:        imagall.cpp
 // Purpose:     wxImage access all handler
 // Author:      Sylvain Bougnoux
-// RCS-ID:      $Id: imagall.cpp,v 1.3 1999/11/28 17:18:25 RR Exp $
+// RCS-ID:      $Id: imagall.cpp,v 1.8.2.1 2002/12/19 12:24:47 VS Exp $
 // Copyright:   (c) Sylvain Bougnoux
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -25,6 +25,8 @@
 
 #include "wx/image.h"
 
+#if wxUSE_IMAGE
+
 //-----------------------------------------------------------------------------
 // This function allows dynamic access to all image handlers compile within
 // the library. This function should be in a separate file as some compilers
@@ -50,4 +52,17 @@ void wxInitAllImageHandlers()
 #if wxUSE_PCX
   wxImage::AddHandler( new wxPCXHandler );
 #endif
+#if wxUSE_IFF
+  wxImage::AddHandler( new wxIFFHandler );
+#endif
+#if wxUSE_XPM && (defined(__WXGTK__) || defined(__WXMOTIF__))
+  wxImage::AddHandler( new wxXPMHandler );
+#endif
+#if wxUSE_ICO_CUR
+  wxImage::AddHandler( new wxICOHandler );
+  wxImage::AddHandler( new wxCURHandler );
+  wxImage::AddHandler( new wxANIHandler );
+#endif
 }
+
+#endif // wxUSE_IMAGE
