@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     29/01/98
-// RCS-ID:      $Id: log.cpp,v 1.93.2.3 2001/04/21 01:34:57 VZ Exp $
+// RCS-ID:      $Id: log.cpp,v 1.128.2.3 2003/06/14 16:45:30 VZ Exp $
 // Copyright:   (c) 1998 Vadim Zeitlin <zeitlin@dptmaths.ens-cachan.fr>
 // Licence:     wxWindows license
 /////////////////////////////////////////////////////////////////////////////
@@ -36,10 +36,10 @@
 
     #if wxUSE_GUI
         #include "wx/window.h"
+        #include "wx/msgdlg.h"
         #ifdef __WXMSW__
             #include "wx/msw/private.h"
         #endif
-        #include "wx/msgdlg.h"
     #endif
 #endif //WX_PRECOMP
 
@@ -50,16 +50,20 @@
 #include  "wx/log.h"
 #include  "wx/thread.h"
 
+#if wxUSE_LOG
+
 // other standard headers
 #include  <errno.h>
 #include  <stdlib.h>
 #include  <time.h>
 
-#ifdef  __WXMSW__
+#if defined(__WXMSW__)
   #include  "wx/msw/private.h"      // includes windows.h for OutputDebugString
-#else   //Unix
-  #include  <signal.h>
-#endif  //Win/Unix
+#endif
+
+#if defined(__WXMAC__)
+  #include  "wx/mac/private.h"  // includes mac headers
+#endif
 
 // ----------------------------------------------------------------------------
 // non member functions
@@ -85,7 +89,10 @@
 #define LOG_BUFFER_SIZE   (4096)
 
 // static buffer for error messages
-static wxChar s_szBuf[LOG_BUFFER_SIZE];
+static wxChar   s_szBufStatic[LOG_BUFFER_SIZE];
+
+static wxChar  *s_szBuf     = s_szBufStatic;
+static size_t   s_szBufSize = WXSIZEOF( s_szBufStatic );
 
 #if wxUSE_THREADS
 
@@ -107,85 +114,142 @@ static inline bool IsLoggingEnabled()
 //     macros and not all compilers inline vararg functions.
 // ----------------------------------------------------------------------------
 
+// wrapper for wxVsnprintf(s_szBuf) which always NULL-terminates it
+static inline void PrintfInLogBug(const wxChar *szFormat, va_list argptr)
+{
+    if ( wxVsnprintf(s_szBuf, s_szBufSize, szFormat, argptr) < 0 )
+    {
+        // must NUL-terminate it manually
+        s_szBuf[s_szBufSize - 1] = _T('\0');
+    }
+    //else: NUL-terminated by vsnprintf()
+}
+
 // generic log function
-void wxLogGeneric(wxLogLevel level, const wxChar *szFormat, ...)
+void wxVLogGeneric(wxLogLevel level, const wxChar *szFormat, va_list argptr)
 {
     if ( IsLoggingEnabled() ) {
         wxCRIT_SECT_LOCKER(locker, gs_csLogBuf);
 
-        va_list argptr;
-        va_start(argptr, szFormat);
-        wxVsnprintf(s_szBuf, WXSIZEOF(s_szBuf), szFormat, argptr);
-        va_end(argptr);
+        PrintfInLogBug(szFormat, argptr);
 
         wxLog::OnLog(level, s_szBuf, time(NULL));
     }
 }
 
+void wxLogGeneric(wxLogLevel level, const wxChar *szFormat, ...)
+{
+    va_list argptr;
+    va_start(argptr, szFormat);
+    wxVLogGeneric(level, szFormat, argptr);
+    va_end(argptr);
+}
+
 #define IMPLEMENT_LOG_FUNCTION(level)                               \
-  void wxLog##level(const wxChar *szFormat, ...)                    \
+  void wxVLog##level(const wxChar *szFormat, va_list argptr)        \
   {                                                                 \
     if ( IsLoggingEnabled() ) {                                     \
       wxCRIT_SECT_LOCKER(locker, gs_csLogBuf);                      \
                                                                     \
-      va_list argptr;                                               \
-      va_start(argptr, szFormat);                                   \
-      wxVsnprintf(s_szBuf, WXSIZEOF(s_szBuf), szFormat, argptr);    \
-      va_end(argptr);                                               \
+      PrintfInLogBug(szFormat, argptr);                             \
                                                                     \
       wxLog::OnLog(wxLOG_##level, s_szBuf, time(NULL));             \
     }                                                               \
+  }                                                                 \
+                                                                    \
+  void wxLog##level(const wxChar *szFormat, ...)                    \
+  {                                                                 \
+    va_list argptr;                                                 \
+    va_start(argptr, szFormat);                                     \
+    wxVLog##level(szFormat, argptr);                                \
+    va_end(argptr);                                                 \
   }
 
-IMPLEMENT_LOG_FUNCTION(FatalError)
 IMPLEMENT_LOG_FUNCTION(Error)
 IMPLEMENT_LOG_FUNCTION(Warning)
 IMPLEMENT_LOG_FUNCTION(Message)
 IMPLEMENT_LOG_FUNCTION(Info)
 IMPLEMENT_LOG_FUNCTION(Status)
 
+void wxSafeShowMessage(const wxString& title, const wxString& text)
+{
+#ifdef __WINDOWS__
+    ::MessageBox(NULL, text, title, MB_OK | MB_ICONSTOP);
+#else
+    wxFprintf(stderr, _T("%s: %s\n"), title.c_str(), text.c_str());
+#endif
+}
+
+// fatal errors can't be suppressed nor handled by the custom log target and
+// always terminate the program
+void wxVLogFatalError(const wxChar *szFormat, va_list argptr)
+{
+    wxVsnprintf(s_szBuf, s_szBufSize, szFormat, argptr);
+
+    wxSafeShowMessage(_T("Fatal Error"), s_szBuf);
+
+    abort();
+}
+
+void wxLogFatalError(const wxChar *szFormat, ...)
+{
+    va_list argptr;
+    va_start(argptr, szFormat);
+    wxVLogFatalError(szFormat, argptr);
+    va_end(argptr);
+}
+
 // same as info, but only if 'verbose' mode is on
-void wxLogVerbose(const wxChar *szFormat, ...)
+void wxVLogVerbose(const wxChar *szFormat, va_list argptr)
 {
     if ( IsLoggingEnabled() ) {
         wxLog *pLog = wxLog::GetActiveTarget();
         if ( pLog != NULL && pLog->GetVerbose() ) {
             wxCRIT_SECT_LOCKER(locker, gs_csLogBuf);
 
-            va_list argptr;
-            va_start(argptr, szFormat);
-            wxVsnprintf(s_szBuf, WXSIZEOF(s_szBuf), szFormat, argptr);
-            va_end(argptr);
+            wxVsnprintf(s_szBuf, s_szBufSize, szFormat, argptr);
 
             wxLog::OnLog(wxLOG_Info, s_szBuf, time(NULL));
         }
     }
 }
 
+void wxLogVerbose(const wxChar *szFormat, ...)
+{
+    va_list argptr;
+    va_start(argptr, szFormat);
+    wxVLogVerbose(szFormat, argptr);
+    va_end(argptr);
+}
+
 // debug functions
 #ifdef __WXDEBUG__
 #define IMPLEMENT_LOG_DEBUG_FUNCTION(level)                         \
-  void wxLog##level(const wxChar *szFormat, ...)                    \
+  void wxVLog##level(const wxChar *szFormat, va_list argptr)        \
   {                                                                 \
     if ( IsLoggingEnabled() ) {                                     \
       wxCRIT_SECT_LOCKER(locker, gs_csLogBuf);                      \
                                                                     \
-      va_list argptr;                                               \
-      va_start(argptr, szFormat);                                   \
-      wxVsnprintf(s_szBuf, WXSIZEOF(s_szBuf), szFormat, argptr);    \
-      va_end(argptr);                                               \
+      wxVsnprintf(s_szBuf, s_szBufSize, szFormat, argptr);    \
                                                                     \
       wxLog::OnLog(wxLOG_##level, s_szBuf, time(NULL));             \
     }                                                               \
+  }                                                                 \
+  void wxLog##level(const wxChar *szFormat, ...)                    \
+  {                                                                 \
+    va_list argptr;                                                 \
+    va_start(argptr, szFormat);                                     \
+    wxVLog##level(szFormat, argptr);                                \
+    va_end(argptr);                                                 \
   }
 
-  void wxLogTrace(const wxChar *mask, const wxChar *szFormat, ...)
+  void wxVLogTrace(const wxChar *mask, const wxChar *szFormat, va_list argptr)
   {
     if ( IsLoggingEnabled() && wxLog::IsAllowedTraceMask(mask) ) {
       wxCRIT_SECT_LOCKER(locker, gs_csLogBuf);
 
       wxChar *p = s_szBuf;
-      size_t len = WXSIZEOF(s_szBuf);
+      size_t len = s_szBufSize;
       wxStrncpy(s_szBuf, _T("("), len);
       len -= 1; // strlen("(")
       p += 1;
@@ -198,16 +262,21 @@ void wxLogVerbose(const wxChar *szFormat, ...)
       len -= 2;
       p += 2;
 
-      va_list argptr;
-      va_start(argptr, szFormat);
       wxVsnprintf(p, len, szFormat, argptr);
-      va_end(argptr);
 
       wxLog::OnLog(wxLOG_Trace, s_szBuf, time(NULL));
     }
   }
 
-  void wxLogTrace(wxTraceMask mask, const wxChar *szFormat, ...)
+  void wxLogTrace(const wxChar *mask, const wxChar *szFormat, ...)
+  {
+    va_list argptr;
+    va_start(argptr, szFormat);
+    wxVLogTrace(mask, szFormat, argptr);
+    va_end(argptr);
+  }
+
+  void wxVLogTrace(wxTraceMask mask, const wxChar *szFormat, va_list argptr)
   {
     // we check that all of mask bits are set in the current mask, so
     // that wxLogTrace(wxTraceRefCount | wxTraceOle) will only do something
@@ -215,15 +284,20 @@ void wxLogVerbose(const wxChar *szFormat, ...)
     if ( IsLoggingEnabled() && ((wxLog::GetTraceMask() & mask) == mask) ) {
       wxCRIT_SECT_LOCKER(locker, gs_csLogBuf);
 
-      va_list argptr;
-      va_start(argptr, szFormat);
-      wxVsnprintf(s_szBuf, WXSIZEOF(s_szBuf), szFormat, argptr);
-      va_end(argptr);
+      wxVsnprintf(s_szBuf, s_szBufSize, szFormat, argptr);
 
       wxLog::OnLog(wxLOG_Trace, s_szBuf, time(NULL));
     }
   }
 
+  void wxLogTrace(wxTraceMask mask, const wxChar *szFormat, ...)
+  {
+    va_list argptr;
+    va_start(argptr, szFormat);
+    wxVLogTrace(mask, szFormat, argptr);
+    va_end(argptr);
+  }
+
 #else // release
   #define IMPLEMENT_LOG_DEBUG_FUNCTION(level)
 #endif
@@ -240,39 +314,49 @@ void wxLogSysErrorHelper(long lErrCode)
     wxChar szErrMsg[LOG_BUFFER_SIZE / 2];
     wxSnprintf(szErrMsg, WXSIZEOF(szErrMsg),
                _(" (error %ld: %s)"), lErrCode, wxSysErrorMsg(lErrCode));
-    wxStrncat(s_szBuf, szErrMsg, WXSIZEOF(s_szBuf) - wxStrlen(s_szBuf));
+    wxStrncat(s_szBuf, szErrMsg, s_szBufSize - wxStrlen(s_szBuf));
 
     wxLog::OnLog(wxLOG_Error, s_szBuf, time(NULL));
 }
 
-void WXDLLEXPORT wxLogSysError(const wxChar *szFormat, ...)
+void WXDLLEXPORT wxVLogSysError(const wxChar *szFormat, va_list argptr)
 {
     if ( IsLoggingEnabled() ) {
         wxCRIT_SECT_LOCKER(locker, gs_csLogBuf);
 
-        va_list argptr;
-        va_start(argptr, szFormat);
-        wxVsnprintf(s_szBuf, WXSIZEOF(s_szBuf), szFormat, argptr);
-        va_end(argptr);
+        wxVsnprintf(s_szBuf, s_szBufSize, szFormat, argptr);
 
         wxLogSysErrorHelper(wxSysErrorCode());
     }
 }
 
-void WXDLLEXPORT wxLogSysError(long lErrCode, const wxChar *szFormat, ...)
+void WXDLLEXPORT wxLogSysError(const wxChar *szFormat, ...)
+{
+    va_list argptr;
+    va_start(argptr, szFormat);
+    wxVLogSysError(szFormat, argptr);
+    va_end(argptr);
+}
+
+void WXDLLEXPORT wxVLogSysError(long lErrCode, const wxChar *szFormat, va_list argptr)
 {
     if ( IsLoggingEnabled() ) {
         wxCRIT_SECT_LOCKER(locker, gs_csLogBuf);
 
-        va_list argptr;
-        va_start(argptr, szFormat);
-        wxVsnprintf(s_szBuf, WXSIZEOF(s_szBuf), szFormat, argptr);
-        va_end(argptr);
+        wxVsnprintf(s_szBuf, s_szBufSize, szFormat, argptr);
 
         wxLogSysErrorHelper(lErrCode);
     }
 }
 
+void WXDLLEXPORT wxLogSysError(long lErrCode, const wxChar *szFormat, ...)
+{
+    va_list argptr;
+    va_start(argptr, szFormat);
+    wxVLogSysError(lErrCode, szFormat, argptr);
+    va_end(argptr);
+}
+
 // ----------------------------------------------------------------------------
 // wxLog class implementation
 // ----------------------------------------------------------------------------
@@ -280,13 +364,24 @@ void WXDLLEXPORT wxLogSysError(long lErrCode, const wxChar *szFormat, ...)
 wxLog::wxLog()
 {
     m_bHasMessages = FALSE;
+}
 
-    // enable verbose messages by default in the debug builds
-#ifdef __WXDEBUG__
-    m_bVerbose = TRUE;
-#else // release
-    m_bVerbose = FALSE;
-#endif // debug/release
+wxChar *wxLog::SetLogBuffer( wxChar *buf, size_t size)
+{
+    wxChar *oldbuf = s_szBuf;
+
+    if( buf == 0 )
+    {
+        s_szBuf = s_szBufStatic;
+        s_szBufSize = WXSIZEOF( s_szBufStatic );
+    }
+    else
+    {
+        s_szBuf = buf;
+        s_szBufSize = size;
+    }
+
+    return (oldbuf == s_szBufStatic ) ? 0 : oldbuf;
 }
 
 wxLog *wxLog::GetActiveTarget()
@@ -327,6 +422,16 @@ wxLog *wxLog::SetActiveTarget(wxLog *pLogger)
     return pOldLogger;
 }
 
+void wxLog::DontCreateOnDemand()
+{
+    ms_bAutoCreate = FALSE;
+
+    // this is usually called at the end of the program and we assume that it
+    // is *always* called at the end - so we free memory here to avoid false
+    // memory leak reports from wxWin  memory tracking code
+    ClearTraceMasks();
+}
+
 void wxLog::RemoveTraceMask(const wxString& str)
 {
     int index = ms_aTraceMasks.Index(str);
@@ -334,6 +439,11 @@ void wxLog::RemoveTraceMask(const wxString& str)
         ms_aTraceMasks.Remove((size_t)index);
 }
 
+void wxLog::ClearTraceMasks()
+{
+    ms_aTraceMasks.Clear();
+}
+
 void wxLog::TimeStamp(wxString *str)
 {
     if ( ms_timestamp )
@@ -395,7 +505,8 @@ void wxLog::DoLogString(const wxChar *WXUNUSED(szString), time_t WXUNUSED(t))
 
 void wxLog::Flush()
 {
-    // do nothing
+    // remember that we don't have any more messages to show
+    m_bHasMessages = FALSE;
 }
 
 // ----------------------------------------------------------------------------
@@ -410,6 +521,192 @@ wxLogStderr::wxLogStderr(FILE *fp)
         m_fp = fp;
 }
 
+#if defined(__WXMAC__) && !defined(__DARWIN__) && defined(__MWERKS__) && (__MWERKS__ >= 0x2400)
+
+// MetroNub stuff doesn't seem to work in CodeWarrior 5.3 Carbon builds...
+
+#ifndef __MetroNubUtils__
+#include "MetroNubUtils.h"
+#endif
+
+#ifdef __cplusplus
+    extern "C" {
+#endif
+
+#ifndef __GESTALT__
+#include <Gestalt.h>
+#endif
+
+#ifndef true
+#define true 1
+#endif
+
+#ifndef false
+#define false 0
+#endif
+
+#if TARGET_API_MAC_CARBON
+
+    #include <CodeFragments.h>
+
+    EXTERN_API_C( long )
+    CallUniversalProc(UniversalProcPtr theProcPtr, ProcInfoType procInfo, ...);
+
+    ProcPtr gCallUniversalProc_Proc = NULL;
+
+#endif
+
+static MetroNubUserEntryBlock*    gMetroNubEntry = NULL;
+
+static long fRunOnce = false;
+
+Boolean IsCompatibleVersion(short inVersion);
+
+/* ---------------------------------------------------------------------------
+        IsCompatibleVersion
+   --------------------------------------------------------------------------- */
+
+Boolean IsCompatibleVersion(short inVersion)
+{
+    Boolean result = false;
+
+    if (fRunOnce)
+    {
+        MetroNubUserEntryBlock* block = (MetroNubUserEntryBlock *)result;
+
+        result = (inVersion <= block->apiHiVersion);
+    }
+
+    return result;
+}
+
+/* ---------------------------------------------------------------------------
+        IsMetroNubInstalled
+   --------------------------------------------------------------------------- */
+
+Boolean IsMetroNubInstalled()
+{
+    if (!fRunOnce)
+    {
+        long result, value;
+
+        fRunOnce = true;
+        gMetroNubEntry = NULL;
+
+        if (Gestalt(gestaltSystemVersion, &value) == noErr && value < 0x1000)
+        {
+            /* look for MetroNub's Gestalt selector */
+            if (Gestalt(kMetroNubUserSignature, &result) == noErr)
+            {
+
+            #if TARGET_API_MAC_CARBON
+                if (gCallUniversalProc_Proc == NULL)
+                {
+                    CFragConnectionID   connectionID;
+                    Ptr                 mainAddress;
+                    Str255              errorString;
+                    ProcPtr             symbolAddress;
+                    OSErr               err;
+                    CFragSymbolClass    symbolClass;
+
+                    symbolAddress = NULL;
+                    err = GetSharedLibrary("\pInterfaceLib", kPowerPCCFragArch, kFindCFrag,
+                                           &connectionID, &mainAddress, errorString);
+
+                    if (err != noErr)
+                    {
+                        gCallUniversalProc_Proc = NULL;
+                        goto end;
+                    }
+
+                    err = FindSymbol(connectionID, "\pCallUniversalProc",
+                                    (Ptr *) &gCallUniversalProc_Proc, &symbolClass);
+
+                    if (err != noErr)
+                    {
+                        gCallUniversalProc_Proc = NULL;
+                        goto end;
+                    }
+                }
+            #endif
+
+                {
+                    MetroNubUserEntryBlock* block = (MetroNubUserEntryBlock *)result;
+
+                    /* make sure the version of the API is compatible */
+                    if (block->apiLowVersion <= kMetroNubUserAPIVersion &&
+                        kMetroNubUserAPIVersion <= block->apiHiVersion)
+                        gMetroNubEntry = block;        /* success! */
+                }
+
+            }
+        }
+    }
+
+end:
+
+#if TARGET_API_MAC_CARBON
+    return (gMetroNubEntry != NULL && gCallUniversalProc_Proc != NULL);
+#else
+    return (gMetroNubEntry != NULL);
+#endif
+}
+
+/* ---------------------------------------------------------------------------
+        IsMWDebuggerRunning                                            [v1 API]
+   --------------------------------------------------------------------------- */
+
+Boolean IsMWDebuggerRunning()
+{
+    if (IsMetroNubInstalled())
+        return CallIsDebuggerRunningProc(gMetroNubEntry->isDebuggerRunning);
+    else
+        return false;
+}
+
+/* ---------------------------------------------------------------------------
+        AmIBeingMWDebugged                                            [v1 API]
+   --------------------------------------------------------------------------- */
+
+Boolean AmIBeingMWDebugged()
+{
+    if (IsMetroNubInstalled())
+        return CallAmIBeingDebuggedProc(gMetroNubEntry->amIBeingDebugged);
+    else
+        return false;
+}
+
+/* ---------------------------------------------------------------------------
+        UserSetWatchPoint                                            [v2 API]
+   --------------------------------------------------------------------------- */
+
+OSErr UserSetWatchPoint (Ptr address, long length, WatchPointIDT* watchPointID)
+{
+    if (IsMetroNubInstalled() && IsCompatibleVersion(kMetroNubUserAPIVersion))
+        return CallUserSetWatchPointProc(gMetroNubEntry->userSetWatchPoint,
+                                         address, length, watchPointID);
+    else
+        return errProcessIsNotClient;
+}
+
+/* ---------------------------------------------------------------------------
+        ClearWatchPoint                                                [v2 API]
+   --------------------------------------------------------------------------- */
+
+OSErr ClearWatchPoint (WatchPointIDT watchPointID)
+{
+    if (IsMetroNubInstalled() && IsCompatibleVersion(kMetroNubUserAPIVersion))
+        return CallClearWatchPointProc(gMetroNubEntry->clearWatchPoint, watchPointID);
+    else
+        return errProcessIsNotClient;
+}
+
+#ifdef __cplusplus
+    }
+#endif
+
+#endif // defined(__WXMAC__) && !defined(__DARWIN__) && (__MWERKS__ >= 0x2400)
+
 void wxLogStderr::DoLogString(const wxChar *szString, time_t WXUNUSED(t))
 {
     wxString str;
@@ -422,13 +719,36 @@ void wxLogStderr::DoLogString(const wxChar *szString, time_t WXUNUSED(t))
 
     // under Windows, programs usually don't have stderr at all, so show the
     // messages also under debugger - unless it's a console program
-#if defined(__WXMSW__) && wxUSE_GUI
+#if defined(__WXMSW__) && wxUSE_GUI && !defined(__WXMICROWIN__)
     str += wxT("\r\n") ;
     OutputDebugString(str.c_str());
 #endif // MSW
-#if defined(__WXMAC__) && wxUSE_GUI
-    debugstr(str + wxT("\r\n"));
-#endif // MSW
+#if defined(__WXMAC__) && !defined(__DARWIN__) && wxUSE_GUI
+    Str255 pstr ;
+    strcpy( (char*) pstr , str.c_str() ) ;
+    strcat( (char*) pstr , ";g" ) ;
+    c2pstr( (char*) pstr ) ;
+
+    Boolean running = false ;
+
+#if defined(__MWERKS__) && (__MWERKS__ >= 0x2400)
+
+    if ( IsMWDebuggerRunning() && AmIBeingMWDebugged() )
+    {
+        running = true ;
+    }
+
+#endif
+
+    if (running)
+    {
+#ifdef __powerc
+        DebugStr(pstr);
+#else
+        SysBreakStr(pstr);
+#endif
+    }
+#endif // Mac
 }
 
 // ----------------------------------------------------------------------------
@@ -436,20 +756,94 @@ void wxLogStderr::DoLogString(const wxChar *szString, time_t WXUNUSED(t))
 // ----------------------------------------------------------------------------
 
 #if wxUSE_STD_IOSTREAM
-wxLogStream::wxLogStream(ostream *ostr)
+wxLogStream::wxLogStream(wxSTD ostream *ostr)
 {
     if ( ostr == NULL )
-        m_ostr = &cerr;
+        m_ostr = &wxSTD cerr;
     else
         m_ostr = ostr;
 }
 
 void wxLogStream::DoLogString(const wxChar *szString, time_t WXUNUSED(t))
 {
-    (*m_ostr) << wxConvertWX2MB(szString) << endl;
+    wxString str;
+    TimeStamp(&str);
+    (*m_ostr) << str << wxConvertWX2MB(szString) << wxSTD endl;
 }
 #endif // wxUSE_STD_IOSTREAM
 
+// ----------------------------------------------------------------------------
+// wxLogChain
+// ----------------------------------------------------------------------------
+
+wxLogChain::wxLogChain(wxLog *logger)
+{
+    m_bPassMessages = TRUE;
+
+    m_logNew = logger;
+    m_logOld = wxLog::SetActiveTarget(this);
+}
+
+wxLogChain::~wxLogChain()
+{
+    delete m_logOld;
+
+    if ( m_logNew != this )
+        delete m_logNew;
+}
+
+void wxLogChain::SetLog(wxLog *logger)
+{
+    if ( m_logNew != this )
+        delete m_logNew;
+
+    m_logNew = logger;
+}
+
+void wxLogChain::Flush()
+{
+    if ( m_logOld )
+        m_logOld->Flush();
+
+    // be careful to avoid inifinite recursion
+    if ( m_logNew && m_logNew != this )
+        m_logNew->Flush();
+}
+
+void wxLogChain::DoLog(wxLogLevel level, const wxChar *szString, time_t t)
+{
+    // let the previous logger show it
+    if ( m_logOld && IsPassingMessages() )
+    {
+        // bogus cast just to access protected DoLog
+        ((wxLogChain *)m_logOld)->DoLog(level, szString, t);
+    }
+
+    if ( m_logNew && m_logNew != this )
+    {
+        // as above...
+        ((wxLogChain *)m_logNew)->DoLog(level, szString, t);
+    }
+}
+
+// ----------------------------------------------------------------------------
+// wxLogPassThrough
+// ----------------------------------------------------------------------------
+
+#ifdef __VISUALC__
+    // "'this' : used in base member initializer list" - so what?
+    #pragma warning(disable:4355)
+#endif // VC++
+
+wxLogPassThrough::wxLogPassThrough()
+                : wxLogChain(this)
+{
+}
+
+#ifdef __VISUALC__
+    #pragma warning(default:4355)
+#endif // VC++
+
 // ============================================================================
 // Global functions/variables
 // ============================================================================
@@ -461,6 +855,9 @@ void wxLogStream::DoLogString(const wxChar *szString, time_t WXUNUSED(t))
 wxLog          *wxLog::ms_pLogger      = (wxLog *)NULL;
 bool            wxLog::ms_doLog        = TRUE;
 bool            wxLog::ms_bAutoCreate  = TRUE;
+bool            wxLog::ms_bVerbose     = FALSE;
+
+wxLogLevel      wxLog::ms_logLevel     = wxLOG_Max;  // log everything by default
 
 size_t          wxLog::ms_suspendCount = 0;
 
@@ -517,7 +914,7 @@ static void wxLogWrap(FILE *f, const char *pszPrefix, const char *psz)
 // get error code from syste
 unsigned long wxSysErrorCode()
 {
-#ifdef  __WXMSW__
+#if defined(__WXMSW__) && !defined(__WXMICROWIN__)
 #ifdef  __WIN32__
     return ::GetLastError();
 #else   //WIN16
@@ -535,7 +932,7 @@ const wxChar *wxSysErrorMsg(unsigned long nErrCode)
     if ( nErrCode == 0 )
         nErrCode = wxSysErrorCode();
 
-#ifdef  __WXMSW__
+#if defined(__WXMSW__) && !defined(__WXMICROWIN__)
 #ifdef  __WIN32__
     static wxChar s_szBuf[LOG_BUFFER_SIZE / 2];
 
@@ -548,17 +945,23 @@ const wxChar *wxSysErrorMsg(unsigned long nErrCode)
             0, NULL);
 
     // copy it to our buffer and free memory
-    wxStrncpy(s_szBuf, (const wxChar *)lpMsgBuf, WXSIZEOF(s_szBuf) - 1);
-    s_szBuf[WXSIZEOF(s_szBuf) - 1] = wxT('\0');
-    LocalFree(lpMsgBuf);
+    if( lpMsgBuf != 0 ) {
+        wxStrncpy(s_szBuf, (const wxChar *)lpMsgBuf, WXSIZEOF(s_szBuf) - 1);
+        s_szBuf[WXSIZEOF(s_szBuf) - 1] = wxT('\0');
 
-    // returned string is capitalized and ended with '\r\n' - bad
-    s_szBuf[0] = (wxChar)wxTolower(s_szBuf[0]);
-    size_t len = wxStrlen(s_szBuf);
-    if ( len > 0 ) {
-        // truncate string
-        if ( s_szBuf[len - 2] == wxT('\r') )
-            s_szBuf[len - 2] = wxT('\0');
+        LocalFree(lpMsgBuf);
+
+        // returned string is capitalized and ended with '\r\n' - bad
+        s_szBuf[0] = (wxChar)wxTolower(s_szBuf[0]);
+        size_t len = wxStrlen(s_szBuf);
+        if ( len > 0 ) {
+            // truncate string
+            if ( s_szBuf[len - 2] == wxT('\r') )
+                s_szBuf[len - 2] = wxT('\0');
+        }
+    }
+    else {
+        s_szBuf[0] = wxT('\0');
     }
 
     return s_szBuf;
@@ -577,113 +980,6 @@ const wxChar *wxSysErrorMsg(unsigned long nErrCode)
 #endif  // Win/Unix
 }
 
-// ----------------------------------------------------------------------------
-// debug helper
-// ----------------------------------------------------------------------------
-
-#ifdef  __WXDEBUG__
-
-// break into the debugger
-void Trap()
-{
-#ifdef __WXMSW__
-    DebugBreak();
-#elif defined(__WXMAC__)
-#if __powerc
-    Debugger();
-#else
-    SysBreak();
-#endif
-#elif defined(__UNIX__)
-    raise(SIGTRAP);
-#else
-    // TODO
-#endif // Win/Unix
-}
-
-// this function is called when an assert fails
-void wxOnAssert(const wxChar *szFile, int nLine, const wxChar *szMsg)
-{
-    // this variable can be set to true to suppress "assert failure" messages
-    static bool s_bNoAsserts = FALSE;
-    static bool s_bInAssert = FALSE;        // FIXME MT-unsafe
-
-    if ( s_bInAssert ) {
-        // He-e-e-e-elp!! we're trapped in endless loop
-        Trap();
-
-        s_bInAssert = FALSE;
-
-        return;
-    }
-
-    s_bInAssert = TRUE;
-
-    wxChar szBuf[LOG_BUFFER_SIZE];
-
-    // make life easier for people using VC++ IDE: clicking on the message
-    // will take us immediately to the place of the failed assert
-    wxSnprintf(szBuf, WXSIZEOF(szBuf),
-#ifdef __VISUALC__
-               wxT("%s(%d): assert failed"),
-#else  // !VC++
-    // make the error message more clear for all the others
-               wxT("Assert failed in file %s at line %d"),
-#endif // VC/!VC
-               szFile, nLine);
-
-    if ( szMsg != NULL ) {
-        wxStrcat(szBuf, wxT(": "));
-        wxStrcat(szBuf, szMsg);
-    }
-    else {
-        wxStrcat(szBuf, wxT("."));
-    }
-
-    if ( !s_bNoAsserts ) {
-        // send it to the normal log destination
-        wxLogDebug(szBuf);
-
-#if wxUSE_GUI || defined(__WXMSW__)
-        // this message is intentionally not translated - it is for
-        // developpers only
-        wxStrcat(szBuf, wxT("\nDo you want to stop the program?\nYou can also choose [Cancel] to suppress further warnings."));
-
-#if wxUSE_GUI
-        switch ( wxMessageBox(szBuf, wxT("Debug"),
-                              wxYES_NO | wxCANCEL | wxICON_STOP ) ) {
-            case wxYES:
-                Trap();
-                break;
-
-            case wxCANCEL:
-                s_bNoAsserts = TRUE;
-                break;
-
-            //case wxNO: nothing to do
-        }
-#else // !GUI, but MSW
-        switch ( ::MessageBox(NULL, szBuf, _T("Debug"),
-                              MB_YESNOCANCEL | MB_ICONSTOP ) ) {
-            case IDYES:
-                Trap();
-                break;
-
-            case IDCANCEL:
-                s_bNoAsserts = TRUE;
-                break;
-
-            //case IDNO: nothing to do
-        }
-#endif // GUI or MSW
-
-#else // !GUI
-        Trap();
-#endif // GUI/!GUI
-    }
-
-    s_bInAssert = FALSE;
-}
-
-#endif  //WXDEBUG
+#endif //wxUSE_LOG
 
+// vi:sts=4:sw=4:et
