@@ -4,9 +4,9 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     04/01/98
-// RCS-ID:      $Id: prntbase.cpp,v 1.39.2.3 2000/07/03 20:22:02 VS Exp $
+// RCS-ID:      $Id: prntbase.cpp,v 1.46.2.5 2003/06/05 12:13:18 JS Exp $
 // Copyright:   (c) Julian Smart and Markus Holzem
-// Licence:       wxWindows license
+// Licence:     wxWindows license
 /////////////////////////////////////////////////////////////////////////////
 
 #ifdef __GNUG__
@@ -36,7 +36,9 @@
 #include "wx/dcmemory.h"
 #include "wx/stattext.h"
 #include "wx/intl.h"
-#endif
+#include "wx/textdlg.h"
+#include "wx/sizer.h"
+#endif // !WX_PRECOMP
 
 #include "wx/prntbase.h"
 #include "wx/dcprint.h"
@@ -101,21 +103,24 @@ void wxPrintAbortDialog::OnCancel(wxCommandEvent& WXUNUSED(event))
     wxPrinterBase::sm_abortWindow = (wxWindow *) NULL;
 }
 
-wxWindow *wxPrinterBase::CreateAbortWindow(wxWindow *parent, wxPrintout *WXUNUSED(printout))
+wxWindow *wxPrinterBase::CreateAbortWindow(wxWindow *parent, wxPrintout * printout)
 {
-    wxPrintAbortDialog *dialog = new wxPrintAbortDialog(parent, _("Printing"), wxPoint(0, 0), wxSize(400, 400), wxDEFAULT_DIALOG_STYLE);
-    (void) new wxStaticText(dialog, -1, _("Please wait..."), wxPoint(5, 5));
+    wxPrintAbortDialog *dialog = new wxPrintAbortDialog(parent, _("Printing ") , wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE);
 
-    wxButton *button = new wxButton(dialog, wxID_CANCEL, _("Cancel"), wxPoint(5, 30));
+    wxBoxSizer *button_sizer = new wxBoxSizer( wxVERTICAL );
+    button_sizer->Add( new wxStaticText(dialog, -1, _("Please wait while printing\n") + printout->GetTitle() ), 0, wxALL, 10 );
+    button_sizer->Add( new wxButton( dialog, wxID_CANCEL, wxT("Cancel") ), 0, wxALL | wxALIGN_CENTER, 10 );
 
-    dialog->Fit();
-    button->Centre(wxHORIZONTAL);
+    dialog->SetAutoLayout( TRUE );
+    dialog->SetSizer( button_sizer );
+
+    button_sizer->Fit(dialog);
+    button_sizer->SetSizeHints (dialog) ;
 
-    dialog->Centre();
     return dialog;
 }
 
-void wxPrinterBase::ReportError(wxWindow *parent, wxPrintout *WXUNUSED(printout), char *message)
+void wxPrinterBase::ReportError(wxWindow *parent, wxPrintout *WXUNUSED(printout), const wxString& message)
 {
     wxMessageBox(message, _("Printing Error"), wxOK, parent);
 }
@@ -145,7 +150,7 @@ wxPrintout::~wxPrintout()
 
 bool wxPrintout::OnBeginDocument(int WXUNUSED(startPage), int WXUNUSED(endPage))
 {
-    return GetDC()->StartDoc(_("Printing"));
+    return GetDC()->StartDoc(_("Printing ") + m_printoutTitle);
 }
 
 void wxPrintout::OnEndDocument()
@@ -183,7 +188,7 @@ wxPreviewCanvas::wxPreviewCanvas(wxPrintPreviewBase *preview, wxWindow *parent,
 wxScrolledWindow(parent, -1, pos, size, style, name)
 {
     m_printPreview = preview;
-    SetBackgroundColour(wxSystemSettings::GetSystemColour(wxSYS_COLOUR_APPWORKSPACE));
+    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_APPWORKSPACE));
 
     SetScrollbars(15, 18, 100, 100);
 }
@@ -213,7 +218,7 @@ void wxPreviewCanvas::OnPaint(wxPaintEvent& WXUNUSED(event))
 // Responds to colour changes, and passes event on to children.
 void wxPreviewCanvas::OnSysColourChanged(wxSysColourChangedEvent& event)
 {
-    SetBackgroundColour(wxSystemSettings::GetSystemColour(wxSYS_COLOUR_APPWORKSPACE));
+    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_APPWORKSPACE));
     Refresh();
 
     // Propagate the event to the non-top-level children
@@ -229,6 +234,9 @@ BEGIN_EVENT_TABLE(wxPreviewControlBar, wxPanel)
     EVT_BUTTON(wxID_PREVIEW_PRINT,    wxPreviewControlBar::OnPrint)
     EVT_BUTTON(wxID_PREVIEW_PREVIOUS, wxPreviewControlBar::OnPreviousButton)
     EVT_BUTTON(wxID_PREVIEW_NEXT,     wxPreviewControlBar::OnNextButton)
+    EVT_BUTTON(wxID_PREVIEW_FIRST,    wxPreviewControlBar::OnFirstButton)
+    EVT_BUTTON(wxID_PREVIEW_LAST,     wxPreviewControlBar::OnLastButton)
+    EVT_BUTTON(wxID_PREVIEW_GOTO,     wxPreviewControlBar::OnGotoButton)
     EVT_CHAR(wxPreviewControlBar::OnChar)
     EVT_CHOICE(wxID_PREVIEW_ZOOM,     wxPreviewControlBar::OnZoom)
     EVT_PAINT(wxPreviewControlBar::OnPaint)
@@ -283,6 +291,12 @@ void wxPreviewControlBar::OnChar(wxKeyEvent &event)
       OnNext(); break;
    case WXK_PRIOR:
       OnPrevious(); break;
+   case WXK_HOME:
+      OnFirst(); break;
+   case WXK_END:
+      OnLast(); break;
+   case WXK_TAB:
+      OnGoto(); break;
    default:
       event.Skip();
    }
@@ -318,6 +332,60 @@ void wxPreviewControlBar::OnPrevious(void)
     }
 }
 
+void wxPreviewControlBar::OnFirst(void)
+{
+    wxPrintPreviewBase *preview = GetPrintPreview();
+    if (preview)
+    {
+        int currentPage = preview->GetMinPage();
+        if (preview->GetPrintout()->HasPage(currentPage))
+        {
+            preview->SetCurrentPage(currentPage);
+        }
+    }
+}
+
+void wxPreviewControlBar::OnLast(void)
+{
+    wxPrintPreviewBase *preview = GetPrintPreview();
+    if (preview)
+    {
+        int currentPage = preview->GetMaxPage();
+        if (preview->GetPrintout()->HasPage(currentPage))
+        {
+            preview->SetCurrentPage(currentPage);
+        }
+    }
+}
+
+void wxPreviewControlBar::OnGoto(void)
+{
+    wxPrintPreviewBase *preview = GetPrintPreview();
+    if (preview)
+    {
+        long currentPage;
+
+        if (preview->GetMinPage() > 0)
+        {
+            wxString strPrompt;
+            wxString strPage;
+
+            strPrompt.Printf( wxT("%d...%d"),
+                preview->GetMinPage(), preview->GetMaxPage());
+            strPage.Printf( wxT("%d"), preview->GetCurrentPage() );
+
+            strPage =
+                wxGetTextFromUser( strPrompt, _("Goto Page"), strPage);
+
+            if ( strPage.ToLong( &currentPage ) )
+                if (preview->GetPrintout()->HasPage(currentPage))
+                {
+                    preview->SetCurrentPage(currentPage);
+                }
+        }
+    }
+}
+
 void wxPreviewControlBar::OnZoom(wxCommandEvent& WXUNUSED(event))
 {
     int zoom = GetZoomControl();
@@ -329,6 +397,7 @@ void wxPreviewControlBar::CreateButtons()
 {
     SetSize(0, 0, 400, 40);
 
+#if wxUSE_CONSTRAINTS
     /*
     #ifdef __WXMSW__
     int fontSize = 9;
@@ -340,7 +409,8 @@ void wxPreviewControlBar::CreateButtons()
       SetFont(buttonFont);
     */
 
-    int buttonWidth = 65;
+    int buttonWidth = 60;
+    int buttonNavigation = 30;
 #ifdef __WXGTK__
     int buttonHeight = -1;
 #else
@@ -368,27 +438,48 @@ void wxPreviewControlBar::CreateButtons()
         x += gap + buttonWidth;
     }
 
+    if (m_buttonFlags & wxPREVIEW_FIRST)
+    {
+        m_firstPageButton = new wxButton(this, wxID_PREVIEW_FIRST, wxT("|<<"), wxPoint(x, y),
+            wxSize(buttonNavigation, buttonHeight));
+        x += gap + buttonNavigation;
+    }
+
     if (m_buttonFlags & wxPREVIEW_PREVIOUS)
     {
         m_previousPageButton = new wxButton(this, wxID_PREVIEW_PREVIOUS, wxT("<<"), wxPoint(x, y),
-            wxSize(buttonWidth, buttonHeight));
-        x += gap + buttonWidth;
+            wxSize(buttonNavigation, buttonHeight));
+        x += gap + buttonNavigation;
     }
 
     if (m_buttonFlags & wxPREVIEW_NEXT)
     {
         m_nextPageButton = new wxButton(this, wxID_PREVIEW_NEXT, wxT(">>"),
-            wxPoint(x, y), wxSize(buttonWidth, buttonHeight));
+            wxPoint(x, y), wxSize(buttonNavigation, buttonHeight));
+        x += gap + buttonNavigation;
+    }
+
+    if (m_buttonFlags & wxPREVIEW_LAST)
+    {
+        m_lastPageButton = new wxButton(this, wxID_PREVIEW_LAST, wxT(">>|"), wxPoint(x, y),
+            wxSize(buttonNavigation, buttonHeight));
+        x += gap + buttonNavigation;
+    }
+
+    if (m_buttonFlags & wxPREVIEW_GOTO)
+    {
+        m_gotoPageButton = new wxButton(this, wxID_PREVIEW_GOTO, _("Goto..."), wxPoint(x, y),
+            wxSize(buttonWidth, buttonHeight));
         x += gap + buttonWidth;
     }
 
     if (m_buttonFlags & wxPREVIEW_ZOOM)
     {
-        static const char *choices[] =
+        static const wxChar *choices[] =
         {
-            "10%", "15%", "20%", "25%", "30%", "35%", "40%", "45%", "50%", "55%",
-            "60%", "65%", "70%", "75%", "80%", "85%", "90%", "95%", "100%", "110%",
-            "120%", "150%", "200%"
+            wxT("10%"), wxT("15%"), wxT("20%"), wxT("25%"), wxT("30%"), wxT("35%"), wxT("40%"), wxT("45%"), wxT("50%"), wxT("55%"),
+            wxT("60%"), wxT("65%"), wxT("70%"), wxT("75%"), wxT("80%"), wxT("85%"), wxT("90%"), wxT("95%"), wxT("100%"), wxT("110%"),
+            wxT("120%"), wxT("150%"), wxT("200%")
         };
 
         int n = WXSIZEOF(choices);
@@ -409,14 +500,77 @@ void wxPreviewControlBar::CreateButtons()
         SetZoomControl(m_printPreview->GetZoom());
     }
 
-    //  m_closeButton->SetDefault();
+#else
+
+    // Use sizers (backported from wxWindows CVS head).
+    wxBoxSizer *item0 = new wxBoxSizer( wxHORIZONTAL );
+    
+    int smallButtonWidth = 45;
+    
+    m_closeButton = new wxButton( this, wxID_PREVIEW_CLOSE, _("&Close"), wxDefaultPosition, wxDefaultSize, 0 );
+    item0->Add( m_closeButton, 0, wxALIGN_CENTRE|wxALL, 5 );
+    
+    if (m_buttonFlags & wxPREVIEW_PRINT)
+    {
+        m_printButton = new wxButton( this, wxID_PREVIEW_PRINT, _("&Print..."), wxDefaultPosition, wxDefaultSize, 0 );
+        item0->Add( m_printButton, 0, wxALIGN_CENTRE|wxALL, 5 );
+    }
+    
+    if (m_buttonFlags & wxPREVIEW_FIRST)
+    {
+        m_firstPageButton = new wxButton( this, wxID_PREVIEW_FIRST, _("|<<"), wxDefaultPosition, wxSize(smallButtonWidth,-1), 0 );
+        item0->Add( m_firstPageButton, 0, wxALIGN_CENTRE|wxALL, 5 );
+    }
+    
+    if (m_buttonFlags & wxPREVIEW_PREVIOUS)
+    {
+        m_previousPageButton = new wxButton( this, wxID_PREVIEW_PREVIOUS, _("<<"), wxDefaultPosition, wxSize(smallButtonWidth,-1), 0 );
+        item0->Add( m_previousPageButton, 0, wxALIGN_CENTRE|wxRIGHT|wxTOP|wxBOTTOM, 5 );
+    }
+    
+    if (m_buttonFlags & wxPREVIEW_NEXT)
+    {
+        m_nextPageButton = new wxButton( this, wxID_PREVIEW_NEXT, _(">>"), wxDefaultPosition, wxSize(smallButtonWidth,-1), 0 );
+        item0->Add( m_nextPageButton, 0, wxALIGN_CENTRE|wxRIGHT|wxTOP|wxBOTTOM, 5 );
+    }
+    
+    if (m_buttonFlags & wxPREVIEW_LAST)
+    {
+        m_lastPageButton = new wxButton( this, wxID_PREVIEW_LAST, _(">>|"), wxDefaultPosition, wxSize(smallButtonWidth,-1), 0 );
+        item0->Add( m_lastPageButton, 0, wxALIGN_CENTRE|wxRIGHT|wxTOP|wxBOTTOM, 5 );
+    }
+    
+    if (m_buttonFlags & wxPREVIEW_GOTO)
+    {
+        m_gotoPageButton = new wxButton( this, wxID_PREVIEW_GOTO, _("&Goto..."), wxDefaultPosition, wxDefaultSize, 0 );
+        item0->Add( m_gotoPageButton, 0, wxALIGN_CENTRE|wxALL, 5 );
+    }
+    
+    if (m_buttonFlags & wxPREVIEW_ZOOM)
+    {
+        wxString choices[] = 
+        {
+            wxT("10%"), wxT("15%"), wxT("20%"), wxT("25%"), wxT("30%"), wxT("35%"), wxT("40%"), wxT("45%"), wxT("50%"), wxT("55%"),
+                wxT("60%"), wxT("65%"), wxT("70%"), wxT("75%"), wxT("80%"), wxT("85%"), wxT("90%"), wxT("95%"), wxT("100%"), wxT("110%"),
+                wxT("120%"), wxT("150%"), wxT("200%")
+        };
+        int n = WXSIZEOF(choices);
+        
+        m_zoomControl = new wxChoice( this, wxID_PREVIEW_ZOOM, wxDefaultPosition, wxSize(70,-1), n, choices, 0 );
+        item0->Add( m_zoomControl, 0, wxALIGN_CENTRE|wxALL, 5 );
+        SetZoomControl(m_printPreview->GetZoom());
+    }
+
+    SetSizer(item0);
+    item0->Fit(this);
+#endif
 }
 
 void wxPreviewControlBar::SetZoomControl(int zoom)
 {
-    char buf[20];
-    sprintf(buf, "%d%%", zoom);
-// Someone is calling methods that do no exist in wxChoice!! So I'll just comment out for VA for now
+    wxChar buf[20];
+    wxSprintf( buf, wxT("%d%%"), zoom );
+
     if (m_zoomControl)
         m_zoomControl->SetStringSelection(buf);
 }
@@ -612,7 +766,8 @@ bool wxPrintPreviewBase::SetCurrentPage(int pageNum)
 
     if (m_previewCanvas)
     {
-        RenderPage(pageNum);
+        if (!RenderPage(pageNum))
+            return FALSE;
         m_previewCanvas->Refresh();
     }
     return TRUE;
@@ -623,7 +778,8 @@ bool wxPrintPreviewBase::PaintPage(wxWindow *canvas, wxDC& dc)
     DrawBlankPage(canvas, dc);
 
     if (!m_previewBitmap)
-        RenderPage(m_currentPage);
+        if (!RenderPage(m_currentPage))
+            return FALSE;
 
     if (!m_previewBitmap)
         return FALSE;
@@ -682,8 +838,10 @@ bool wxPrintPreviewBase::RenderPage(int pageNum)
         m_previewBitmap = new wxBitmap((int)actualWidth, (int)actualHeight);
         if (!m_previewBitmap || !m_previewBitmap->Ok())
         {
-            if (m_previewBitmap)
+            if (m_previewBitmap) {
                 delete m_previewBitmap;
+                m_previewBitmap = NULL;
+            }
             wxMessageBox(_("Sorry, not enough memory to create a preview."), _("Print Preview Failure"), wxOK);
             return FALSE;
         }
@@ -713,6 +871,7 @@ bool wxPrintPreviewBase::RenderPage(int pageNum)
         memoryDC.SelectObject(wxNullBitmap);
 
         delete m_previewBitmap;
+        m_previewBitmap = NULL;
         return FALSE;
     }
 
