@@ -1,7 +1,7 @@
 /*
 
 This file was altered for needs of wxWindows.
-$Id: unzip.c,v 1.13.2.3 2000/07/27 22:06:17 VZ Exp $
+$Id: unzip.c,v 1.24 2001/10/29 14:33:46 VZ Exp $
 
 */
 
@@ -12,16 +12,26 @@ $Id: unzip.c,v 1.13.2.3 2000/07/27 22:06:17 VZ Exp $
 
 */
 
-
-
-#include "wx/setup.h"
+#include "wx/platform.h"
 
 #if wxUSE_ZLIB && wxUSE_ZIPSTREAM
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+/*
 #include "zlib.h"
+ normally, the compiler options should contain -I../zlib, but it is
+ apparently not the case for all MSW makefiles and so, unless we use
+ configure (which defines __WX_SETUP_H__) or it is explicitly overridden by
+ the user (who can define wxUSE_ZLIB_H_IN_PATH), we hardcode the path here
+*/
+#if defined(__WXMSW__) && !defined(__WX_SETUP_H__) && !defined(wxUSE_ZLIB_H_IN_PATH)
+   #include "../zlib/zlib.h"
+#else
+   #include "zlib.h"
+#endif
+
 
 /* Not the right solution (paths in makefiles) but... */
 #ifdef __BORLANDC__
@@ -310,13 +320,13 @@ local uLong unzlocal_SearchCentralDir(fin)
 	uLong uBackRead;
 	uLong uMaxBack=0xffff; /* maximum size of global comment */
 	uLong uPosFound=0;
-	
+
 	if (fseek(fin,0,SEEK_END) != 0)
 		return 0;
 
 
 	uSizeFile = ftell( fin );
-	
+
 	if (uMaxBack>uSizeFile)
 		uMaxBack = uSizeFile;
 
@@ -334,7 +344,7 @@ local uLong unzlocal_SearchCentralDir(fin)
 		else
 			uBackRead+=BUFREADCOMMENT;
 		uReadPos = uSizeFile-uBackRead ;
-		
+
 		uReadSize = ((BUFREADCOMMENT+4) < (uSizeFile-uReadPos)) ?
                      (BUFREADCOMMENT+4) : (uSizeFile-uReadPos);
 		if (fseek(fin,uReadPos,SEEK_SET)!=0)
@@ -358,7 +368,7 @@ local uLong unzlocal_SearchCentralDir(fin)
 	return uPosFound;
 }
 
-#ifdef __WXMAC__
+#if defined(__WXMAC__) && !defined(__UNIX__)
 void wxUnix2MacFilename (char *s) ;
 void
 wxUnix2MacFilename (char *s)
@@ -430,10 +440,10 @@ extern unzFile ZEXPORT unzOpen (path)
     if (unz_copyright[0]!=' ')
         return NULL;
 
-#ifdef __WXMAC__
+#if defined(__WXMAC__) && !defined(__UNIX__)
 	strcpy( wxBuffer , path ) ;
 	wxUnix2MacFilename( wxBuffer ) ;
-  fin=fopen(wxBuffer,"rb");
+	fin=fopen(wxBuffer,"rb");
 #else
     fin=fopen(path,"rb");
 #endif
@@ -499,12 +509,12 @@ extern unzFile ZEXPORT unzOpen (path)
 		                    (us.offset_central_dir+us.size_central_dir);
 	us.central_pos = central_pos;
     us.pfile_in_zip_read = NULL;
-	
+
 
 	s=(unz_s*)ALLOC(sizeof(unz_s));
 	*s=us;
-	unzGoToFirstFile((unzFile)s);	
-	return (unzFile)s;	
+	unzGoToFirstFile((unzFile)s);
+	return (unzFile)s;
 }
 
 
@@ -683,7 +693,7 @@ local int unzlocal_GetCurrentFileInfoInternal (file,
 		lSeek -= uSizeRead;
 	}
 
-	
+
 	if ((err==UNZ_OK) && (extraField!=NULL))
 	{
 		uLong uSizeRead ;
@@ -708,7 +718,7 @@ local int unzlocal_GetCurrentFileInfoInternal (file,
 	else
 		lSeek+=file_info.size_file_extra;
 
-	
+
 	if ((err==UNZ_OK) && (szComment!=NULL))
 	{
 		uLong uSizeRead ;
@@ -802,7 +812,7 @@ extern int ZEXPORT unzGoToFirstFile (file)
 extern int ZEXPORT unzGoToNextFile (file)
 	unzFile file;
 {
-	unz_s* s;	
+	unz_s* s;
 	int err;
 
 	if (file==NULL)
@@ -837,20 +847,20 @@ extern int ZEXPORT unzLocateFile (file, szFileName, iCaseSensitivity)
 	const char *szFileName;
 	int iCaseSensitivity;
 {
-	unz_s* s;	
+	unz_s* s;
 	int err;
     const char *c;
     char *c2;
     char szFileName2[UNZ_MAXFILENAMEINZIP+1];
-    
+
 	uLong num_fileSaved;
 	uLong pos_in_central_dirSaved;
 
-    for (c = szFileName, c2 = szFileName2; *c != '\0'; c++, c2++) 
+    for (c = szFileName, c2 = szFileName2; *c != '\0'; c++, c2++)
         if (*c == '\\') *c2 = '/';
         else *c2 = *c;
     *c2 = '\0';
-	
+
 	if (file==NULL)
 		return UNZ_PARAMERROR;
 
@@ -979,7 +989,7 @@ local int unzlocal_CheckCurrentFileCoherencyHeader (s,piSizeVar,
 
 	return err;
 }
-												
+
 /*
   Open for reading data the current file in the zipfile.
   If there is no error and the file is opened, the return value is UNZ_OK.
@@ -1025,7 +1035,7 @@ extern int ZEXPORT unzOpenCurrentFile (file)
 	}
 
 	pfile_in_zip_read_info->stream_initialised=0;
-	
+
 	if ((s->cur_file_info.compression_method!=0) &&
         (s->cur_file_info.compression_method!=Z_DEFLATED))
 		err=UNZ_BADZIPFILE;
@@ -1062,11 +1072,11 @@ extern int ZEXPORT unzOpenCurrentFile (file)
 	pfile_in_zip_read_info->rest_read_uncompressed =
             s->cur_file_info.uncompressed_size ;
 
-	
+
 	pfile_in_zip_read_info->pos_in_zipfile =
             s->cur_file_info_internal.offset_curfile + SIZEZIPLOCALHEADER +
 			  iSizeVar;
-	
+
 	pfile_in_zip_read_info->stream.avail_in = (uInt)0;
 
 
@@ -1111,7 +1121,7 @@ extern int ZEXPORT unzReadCurrentFile  (file, buf, len)
 	pfile_in_zip_read_info->stream.next_out = (Bytef*)buf;
 
 	pfile_in_zip_read_info->stream.avail_out = (uInt)len;
-	
+
 	if (len>pfile_in_zip_read_info->rest_read_uncompressed)
 		pfile_in_zip_read_info->stream.avail_out =
 		  (uInt)pfile_in_zip_read_info->rest_read_uncompressed;
@@ -1136,7 +1146,7 @@ extern int ZEXPORT unzReadCurrentFile  (file, buf, len)
 			pfile_in_zip_read_info->pos_in_zipfile += uReadThis;
 
 			pfile_in_zip_read_info->rest_read_compressed-=uReadThis;
-			
+
 			pfile_in_zip_read_info->stream.next_in =
                 (Bytef*)pfile_in_zip_read_info->read_buffer;
 			pfile_in_zip_read_info->stream.avail_in = (uInt)uReadThis;
@@ -1150,11 +1160,11 @@ extern int ZEXPORT unzReadCurrentFile  (file, buf, len)
 				uDoCopy = pfile_in_zip_read_info->stream.avail_out ;
 			else
 				uDoCopy = pfile_in_zip_read_info->stream.avail_in ;
-				
+
 			for (i=0;i<uDoCopy;i++)
 				*(pfile_in_zip_read_info->stream.next_out+i) =
                         *(pfile_in_zip_read_info->stream.next_in+i);
-					
+
 			pfile_in_zip_read_info->crc32 = crc32(pfile_in_zip_read_info->crc32,
 								pfile_in_zip_read_info->stream.next_out,
 								uDoCopy);
@@ -1186,7 +1196,7 @@ extern int ZEXPORT unzReadCurrentFile  (file, buf, len)
 
 			uTotalOutAfter = pfile_in_zip_read_info->stream.total_out;
 			uOutThis = uTotalOutAfter-uTotalOutBefore;
-			
+
 			pfile_in_zip_read_info->crc32 =
                 crc32(pfile_in_zip_read_info->crc32,bufBefore,
                         (uInt)(uOutThis));
@@ -1244,7 +1254,7 @@ extern int ZEXPORT unzeof (file)
 
 	if (pfile_in_zip_read_info==NULL)
 		return UNZ_PARAMERROR;
-	
+
 	if (pfile_in_zip_read_info->rest_read_uncompressed == 0)
 		return 1;
 	else
@@ -1288,7 +1298,7 @@ extern int ZEXPORT unzGetLocalExtrafield (file,buf,len)
 
 	if (buf==NULL)
 		return (int)size_to_read;
-	
+
 	if (len>size_to_read)
 		read_now = (uInt)size_to_read;
 	else
@@ -1296,7 +1306,7 @@ extern int ZEXPORT unzGetLocalExtrafield (file,buf,len)
 
 	if (read_now==0)
 		return 0;
-	
+
 	if (fseek(pfile_in_zip_read_info->file,
               pfile_in_zip_read_info->offset_local_extrafield +
 			  pfile_in_zip_read_info->pos_local_extrafield,SEEK_SET)!=0)
