@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     01/02/97
-// RCS-ID:      $Id: dde.cpp,v 1.19.2.3 2001/04/12 22:29:30 VZ Exp $
+// RCS-ID:      $Id: dde.cpp,v 1.27.2.1 2005/04/23 19:06:10 JS Exp $
 // Copyright:   (c) Julian Smart and Markus Holzem
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -46,8 +46,8 @@
 #include <windows.h>
 #include <ddeml.h>
 
-#if defined(__TWIN32__) || defined(__GNUWIN32_OLD__)
-    #include "wx/msw/gnuwin32/extra.h"
+#ifdef __WXWINE__
+#define PCONVCONTEXT CONVCONTEXT*
 #endif
 
 // some compilers headers don't define this one (mingw32)
@@ -129,8 +129,6 @@ static wxList wxAtomTable(wxKEY_STRING);
 static wxList wxDDEClientObjects;
 static wxList wxDDEServerObjects;
 
-char *DDEDefaultIPCBuffer = NULL;
-int DDEDefaultIPCBufferSize = 0;
 static bool DDEInitialized = FALSE;
 
 // ----------------------------------------------------------------------------
@@ -189,13 +187,21 @@ extern void wxDDEInitialize()
 
 void wxDDECleanUp()
 {
+    wxDDEClientObjects.DeleteContents(TRUE);
+    wxDDEClientObjects.Clear();
+    wxDDEClientObjects.DeleteContents(FALSE);
+
+    wxDDEServerObjects.DeleteContents(TRUE);
+    wxDDEServerObjects.Clear();
+    wxDDEServerObjects.DeleteContents(FALSE);
+
+    wxAtomTable.Clear();
+
     if ( DDEIdInst != 0 )
     {
         DdeUninitialize(DDEIdInst);
         DDEIdInst = 0;
     }
-
-    delete [] DDEDefaultIPCBuffer;
 }
 
 // ----------------------------------------------------------------------------
@@ -310,6 +316,7 @@ wxDDEServer::~wxDDEServer()
     {
         wxDDEConnection *connection = (wxDDEConnection *)node->Data();
         wxNode *next = node->Next();
+        connection->SetConnected(false);
         connection->OnDisconnect(); // May delete the node implicitly
         node = next;
     }
@@ -458,21 +465,9 @@ bool wxDDEClient::DeleteConnection(WXHCONV conv)
 // wxDDEConnection
 // ----------------------------------------------------------------------------
 
-wxDDEConnection::wxDDEConnection(char *buffer, int size)
+wxDDEConnection::wxDDEConnection(wxChar *buffer, int size)
+     : wxConnectionBase(buffer, size)
 {
-    if (buffer == NULL)
-    {
-        if (DDEDefaultIPCBuffer == NULL)
-            DDEDefaultIPCBuffer = new char[DDEDefaultIPCBufferSize];
-        m_bufPtr = DDEDefaultIPCBuffer;
-        m_bufSize = DDEDefaultIPCBufferSize;
-    }
-    else
-    {
-        m_bufPtr = buffer;
-        m_bufSize = size;
-    }
-
     m_client = NULL;
     m_server = NULL;
 
@@ -481,20 +476,17 @@ wxDDEConnection::wxDDEConnection(char *buffer, int size)
 }
 
 wxDDEConnection::wxDDEConnection()
+     : wxConnectionBase()
 {
     m_hConv = 0;
     m_sendingData = NULL;
     m_server = NULL;
     m_client = NULL;
-    if (DDEDefaultIPCBuffer == NULL)
-        DDEDefaultIPCBuffer = new char[DDEDefaultIPCBufferSize];
-
-    m_bufPtr = DDEDefaultIPCBuffer;
-    m_bufSize = DDEDefaultIPCBufferSize;
 }
 
 wxDDEConnection::~wxDDEConnection()
 {
+    Disconnect();
     if (m_server)
         m_server->GetConnections().DeleteObject(this);
     else
@@ -504,6 +496,9 @@ wxDDEConnection::~wxDDEConnection()
 // Calls that CLIENT can make
 bool wxDDEConnection::Disconnect()
 {
+    if ( !GetConnected() )
+        return true;
+
     DDEDeleteConnection(GetHConv());
 
     bool ok = DdeDisconnect(GetHConv()) != 0;
@@ -512,35 +507,27 @@ bool wxDDEConnection::Disconnect()
         DDELogError(_T("Failed to disconnect from DDE server gracefully"));
     }
 
+    SetConnected( false );  // so we don't try and disconnect again
+
     return ok;
 }
 
 bool wxDDEConnection::Execute(const wxChar *data, int size, wxIPCFormat format)
 {
     DWORD result;
-    if ( size < 0 )
+    if (size < 0)
     {
         size = wxStrlen(data) + 1;
     }
 
-    HDDEDATA hData = DdeCreateDataHandle(DDEIdInst, (LPBYTE)data, size,
-                                         0, 0, format, 0);
-    if ( !hData )
-    {
-        DDELogError(_T("Failed to create DDE data handle"));
-
-        return FALSE;
-    }
-
-    // size == -1 is special and means that we pass a DDE data handle
-    size = -1;
-    bool ok = DdeClientTransaction((LPBYTE)hData, size,
+    bool ok = DdeClientTransaction((LPBYTE)data, size,
                                     GetHConv(),
                                     NULL,
-                                    0, //format,
+                                    format,
                                     XTYP_EXECUTE,
                                     DDE_TIMEOUT,
                                     &result) != 0;
+
     if ( !ok )
     {
         DDELogError(_T("DDE execute request failed"));
@@ -549,9 +536,10 @@ bool wxDDEConnection::Execute(const wxChar *data, int size, wxIPCFormat format)
     return ok;
 }
 
-char *wxDDEConnection::Request(const wxString& item, int *size, wxIPCFormat format)
+wxChar *wxDDEConnection::Request(const wxString& item, int *size, wxIPCFormat format)
 {
     DWORD result;
+
     HSZ atom = DDEGetAtom(item);
 
     HDDEDATA returned_data = DdeClientTransaction(NULL, 0,
@@ -567,14 +555,19 @@ char *wxDDEConnection::Request(const wxString& item, int *size, wxIPCFormat form
         return NULL;
     }
 
-    DWORD len = DdeGetData(returned_data, (LPBYTE)m_bufPtr, m_bufSize, 0);
+    DWORD len = DdeGetData(returned_data, NULL, 0, 0);
+
+    wxChar *data = GetBufferAtLeast( len );
+    wxASSERT_MSG(data != NULL,
+                 _T("Buffer too small in wxDDEConnection::Request") );
+    DdeGetData(returned_data, (LPBYTE)data, len, 0);
 
     DdeFreeDataHandle(returned_data);
 
     if (size)
         *size = (int)len;
 
-    return m_bufPtr;
+    return data;
 }
 
 bool wxDDEConnection::Poke(const wxString& item, wxChar *data, int size, wxIPCFormat format)
@@ -651,7 +644,7 @@ bool wxDDEConnection::Advise(const wxString& item,
 
     HSZ item_atom = DDEGetAtom(item);
     HSZ topic_atom = DDEGetAtom(m_topicName);
-    m_sendingData = data;
+    m_sendingData = data;  // mrf: potential for scope problems here?
     m_dataSize = size;
     m_dataType = format;
 
@@ -724,10 +717,14 @@ _DDECallback(WORD wType,
         case XTYP_DISCONNECT:
             {
                 wxDDEConnection *connection = DDEFindConnection(hConv);
-                if (connection && connection->OnDisconnect())
+                if (connection)
                 {
-                    DDEDeleteConnection(hConv);  // Delete mapping: hConv => connection
-                    return (DDERETURN)(DWORD)TRUE;
+                    connection->SetConnected( false );
+                    if (connection->OnDisconnect())
+                    {
+                        DDEDeleteConnection(hConv);  // Delete mapping: hConv => connection
+                        return (DDERETURN)(DWORD)TRUE;
+                    }
                 }
                 break;
             }
@@ -738,13 +735,18 @@ _DDECallback(WORD wType,
 
                 if (connection)
                 {
-                    DWORD len = DdeGetData(hData,
-                                           (LPBYTE)connection->m_bufPtr,
-                                           connection->m_bufSize,
-                                           0);
+                    DWORD len = DdeGetData(hData, NULL, 0, 0);
+
+                    wxChar *data = connection->GetBufferAtLeast( len );
+                    wxASSERT_MSG(data != NULL,
+                                 _T("Buffer too small in _DDECallback (XTYP_EXECUTE)") );
+
+                    DdeGetData(hData, (LPBYTE)data, len, 0);
+
                     DdeFreeDataHandle(hData);
+
                     if ( connection->OnExecute(connection->m_topicName,
-                                               connection->m_bufPtr,
+                                               data,
                                                (int)len,
                                                (wxIPCFormat) wFmt) )
                     {
@@ -764,7 +766,7 @@ _DDECallback(WORD wType,
                     wxString item_name = DDEStringFromAtom(hsz2);
 
                     int user_size = -1;
-                    char *data = connection->OnRequest(connection->m_topicName,
+                    wxChar *data = connection->OnRequest(connection->m_topicName,
                                                        item_name,
                                                        &user_size,
                                                        (wxIPCFormat) wFmt);
@@ -794,15 +796,19 @@ _DDECallback(WORD wType,
                 {
                     wxString item_name = DDEStringFromAtom(hsz2);
 
-                    DWORD len = DdeGetData(hData,
-                                           (LPBYTE)connection->m_bufPtr,
-                                           connection->m_bufSize,
-                                           0);
+                    DWORD len = DdeGetData(hData, NULL, 0, 0);
+
+                    wxChar *data = connection->GetBufferAtLeast( len );
+                    wxASSERT_MSG(data != NULL,
+                                 _T("Buffer too small in _DDECallback (XTYP_EXECUTE)") );
+
+                    DdeGetData(hData, (LPBYTE)data, len, 0);
+
                     DdeFreeDataHandle(hData);
 
                     connection->OnPoke(connection->m_topicName,
                                        item_name,
-                                       (wxChar*)connection->m_bufPtr,
+                                       data,
                                        (int)len,
                                        (wxIPCFormat) wFmt);
 
@@ -877,14 +883,18 @@ _DDECallback(WORD wType,
                 {
                     wxString item_name = DDEStringFromAtom(hsz2);
 
-                    DWORD len = DdeGetData(hData,
-                                           (LPBYTE)connection->m_bufPtr,
-                                           connection->m_bufSize,
-                                           0);
+                    DWORD len = DdeGetData(hData, NULL, 0, 0);
+
+                    wxChar *data = connection->GetBufferAtLeast( len );
+                    wxASSERT_MSG(data != NULL,
+                                 _T("Buffer too small in _DDECallback (XTYP_ADVDATA)") );
+
+                    DdeGetData(hData, (LPBYTE)data, len, 0);
+
                     DdeFreeDataHandle(hData);
                     if ( connection->OnAdvise(connection->m_topicName,
                                               item_name,
-                                              connection->m_bufPtr,
+                                              data,
                                               (int)len,
                                               (wxIPCFormat) wFmt) )
                     {
