@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     11.06.98
-// RCS-ID:      $Id: notebook.cpp,v 1.48 2000/02/14 18:21:20 VZ Exp $
+// RCS-ID:      $Id: notebook.cpp,v 1.64.2.11 2005/05/14 16:58:36 JS Exp $
 // Copyright:   (c) 1998 Vadim Zeitlin <zeitlin@dptmaths.ens-cachan.fr>
 // Licence:     wxWindows license
 ///////////////////////////////////////////////////////////////////////////////
@@ -17,9 +17,11 @@
 #include "wx/wxprec.h"
 
 #ifdef __BORLANDC__
-#pragma hdrstop
+    #pragma hdrstop
 #endif
 
+#if wxUSE_NOTEBOOK
+
 // wxWindows
 #ifndef WX_PRECOMP
   #include  "wx/string.h"
@@ -30,6 +32,7 @@
 #include  "wx/event.h"
 #include  "wx/control.h"
 #include  "wx/notebook.h"
+#include  "wx/app.h"
 
 #include  "wx/msw/private.h"
 
@@ -40,14 +43,23 @@
 
 #include  <windowsx.h>  // for SetWindowFont
 
-#ifndef __TWIN32__
-    #ifdef __GNUWIN32_OLD__
-        #include "wx/msw/gnuwin32/extra.h"
-    #endif
-#endif
+#include <commctrl.h>
 
-#if defined(__WIN95__) && !(defined(__GNUWIN32_OLD__) || defined(__TWIN32__))
-    #include <commctrl.h>
+#include "wx/msw/winundef.h"
+
+#if wxUSE_UXTHEME
+#include "wx/msw/uxtheme.h"
+
+#include "wx/radiobut.h"
+#include "wx/radiobox.h"
+#include "wx/checkbox.h"
+#include "wx/bmpbuttn.h"
+#include "wx/statline.h"
+#include "wx/statbox.h"
+#include "wx/stattext.h"
+#include "wx/slider.h"
+#include "wx/scrolwin.h"
+#include "wx/panel.h"
 #endif
 
 // ----------------------------------------------------------------------------
@@ -81,6 +93,9 @@
 // event table
 // ----------------------------------------------------------------------------
 
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGED)
+DEFINE_EVENT_TYPE(wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGING)
+
 BEGIN_EVENT_TABLE(wxNotebook, wxControl)
     EVT_NOTEBOOK_PAGE_CHANGED(-1, wxNotebook::OnSelChange)
 
@@ -105,7 +120,7 @@ IMPLEMENT_DYNAMIC_CLASS(wxNotebookEvent, wxNotifyEvent)
 // common part of all ctors
 void wxNotebook::Init()
 {
-  m_pImageList = NULL;
+  m_imageList = NULL;
   m_nSelection = -1;
 }
 
@@ -136,69 +151,73 @@ bool wxNotebook::Create(wxWindow *parent,
                         long style,
                         const wxString& name)
 {
-  // base init
-  if ( !CreateBase(parent, id, pos, size, style, wxDefaultValidator, name) )
-      return FALSE;
+    // Does ComCtl32 support non-top tabs?
+    int verComCtl32 = wxApp::GetComCtl32Version();
+    if ( verComCtl32 < 470 || verComCtl32 >= 600 )
+    {
+        if (style & wxNB_BOTTOM)
+            style &= ~wxNB_BOTTOM;
+        
+        if (style & wxNB_LEFT)
+            style &= ~wxNB_LEFT;
+        
+        if (style & wxNB_RIGHT)
+            style &= ~wxNB_RIGHT;
+    }
+    
+    // base init
+    if ( !CreateControl(parent, id, pos, size, style, wxDefaultValidator, name) )
+        return FALSE;
 
-  // colors and font
-  m_backgroundColour = wxColour(GetSysColor(COLOR_BTNFACE));
-  m_foregroundColour = *wxBLACK;
+    // notebook, so explicitly specify 0 as last parameter
+    if ( !MSWCreateControl(WC_TABCONTROL, _T(""), pos, size,
+                style | wxTAB_TRAVERSAL) )
+        return FALSE;
 
-  // style
-  m_windowStyle = style | wxTAB_TRAVERSAL;
+    SetBackgroundColour(wxColour(::GetSysColor(COLOR_BTNFACE)));
 
-  long tabStyle = WS_CHILD | WS_VISIBLE | WS_TABSTOP | TCS_TABS;
-
-  if (m_windowStyle & wxCLIP_CHILDREN)
-    tabStyle |= WS_CLIPCHILDREN;
-  if ( m_windowStyle & wxTC_MULTILINE )
-    tabStyle |= TCS_MULTILINE;
-  if ( m_windowStyle & wxBORDER )
-    tabStyle &= WS_BORDER;
-  if (m_windowStyle & wxNB_FIXEDWIDTH)
-    tabStyle |= TCS_FIXEDWIDTH ;
-  if (m_windowStyle & wxNB_BOTTOM)
-    tabStyle |= TCS_RIGHT;
-  if (m_windowStyle & wxNB_LEFT)
-    tabStyle |= TCS_VERTICAL;
-  if (m_windowStyle & wxNB_RIGHT)
-    tabStyle |= TCS_VERTICAL|TCS_RIGHT;
-
-
-  if ( !MSWCreate(GetId(), GetParent(), WC_TABCONTROL,
-                  this, NULL, pos.x, pos.y, size.x, size.y,
-                  tabStyle, NULL, 0) )
-  {
-    return FALSE;
-  }
-
-  // Not all compilers recognise SetWindowFont
-  ::SendMessage(GetHwnd(), WM_SETFONT,
-                (WPARAM)::GetStockObject(DEFAULT_GUI_FONT), TRUE);
-
-
-  if ( parent != NULL )
-    parent->AddChild(this);
-
-  SubclassWin(m_hWnd);
-
-  return TRUE;
+    return TRUE;
 }
 
-// dtor
-wxNotebook::~wxNotebook()
+WXDWORD wxNotebook::MSWGetStyle(long style, WXDWORD *exstyle) const
 {
+    WXDWORD tabStyle = wxControl::MSWGetStyle(style, exstyle);
+
+    tabStyle |= WS_TABSTOP | TCS_TABS;
+
+    if ( style & wxNB_MULTILINE )
+        tabStyle |= TCS_MULTILINE;
+    if ( style & wxNB_FIXEDWIDTH )
+        tabStyle |= TCS_FIXEDWIDTH;
+
+    if ( style & wxNB_BOTTOM )
+        tabStyle |= TCS_RIGHT;
+    else if ( style & wxNB_LEFT )
+        tabStyle |= TCS_VERTICAL;
+    else if ( style & wxNB_RIGHT )
+        tabStyle |= TCS_VERTICAL | TCS_RIGHT;
+
+    // ex style
+    if ( exstyle )
+    {
+        // note that we never want to have the default WS_EX_CLIENTEDGE style
+        // as it looks too ugly for the notebooks
+        *exstyle = 0;
+    }
+
+    return tabStyle;
 }
 
 // ----------------------------------------------------------------------------
 // wxNotebook accessors
 // ----------------------------------------------------------------------------
+
 int wxNotebook::GetPageCount() const
 {
   // consistency check
-  wxASSERT( (int)m_aPages.Count() == TabCtrl_GetItemCount(m_hwnd) );
+  wxASSERT( (int)m_pages.Count() == TabCtrl_GetItemCount(m_hwnd) );
 
-  return m_aPages.Count();
+  return m_pages.Count();
 }
 
 int wxNotebook::GetRowCount() const
@@ -210,19 +229,23 @@ int wxNotebook::SetSelection(int nPage)
 {
   wxCHECK_MSG( IS_VALID_PAGE(nPage), -1, wxT("notebook page out of range") );
 
-  ChangePage(m_nSelection, nPage);
+  if ( nPage != m_nSelection )
+  {
+    wxNotebookEvent event(wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGING, m_windowId);
+    event.SetSelection(nPage);
+    event.SetOldSelection(m_nSelection);
+    event.SetEventObject(this);
+    if ( !GetEventHandler()->ProcessEvent(event) || event.IsAllowed() )
+    {
+      // program allows the page change
+      event.SetEventType(wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGED);
+      (void)GetEventHandler()->ProcessEvent(event);
 
-  return TabCtrl_SetCurSel(m_hwnd, nPage);
-}
+      TabCtrl_SetCurSel(m_hwnd, nPage);
+    }
+  }
 
-void wxNotebook::AdvanceSelection(bool bForward)
-{
-  int nSel = GetSelection();
-  int nMax = GetPageCount() - 1;
-  if ( bForward )
-    SetSelection(nSel == nMax ? 0 : nSel + 1);
-  else
-    SetSelection(nSel == 0 ? nMax : nSel - 1);
+  return m_nSelection;
 }
 
 bool wxNotebook::SetPageText(int nPage, const wxString& strText)
@@ -276,8 +299,12 @@ bool wxNotebook::SetPageImage(int nPage, int nImage)
 
 void wxNotebook::SetImageList(wxImageList* imageList)
 {
-  m_pImageList = imageList;
-  TabCtrl_SetImageList(m_hwnd, (HIMAGELIST)imageList->GetHIMAGELIST());
+  wxNotebookBase::SetImageList(imageList);
+
+  if ( imageList )
+  {
+    TabCtrl_SetImageList(m_hwnd, (HIMAGELIST)imageList->GetHIMAGELIST());
+  }
 }
 
 // ----------------------------------------------------------------------------
@@ -315,48 +342,54 @@ void wxNotebook::SetTabSize(const wxSize& sz)
 // wxNotebook operations
 // ----------------------------------------------------------------------------
 
-// remove one page from the notebook
-bool wxNotebook::DeletePage(int nPage)
-{
-  wxCHECK_MSG( IS_VALID_PAGE(nPage), FALSE, wxT("notebook page out of range") );
-
-  if ( m_nSelection == nPage ) {
-      // advance selection backwards - the page being deleted shouldn't be left
-      // selected
-      AdvanceSelection(FALSE);
-  }
-
-  TabCtrl_DeleteItem(m_hwnd, nPage);
-
-  delete m_aPages[nPage];
-  m_aPages.Remove(nPage);
-
-  if ( m_aPages.IsEmpty() ) {
-      // no selection if the notebook became empty
-      m_nSelection = -1;
-  }
-  else
-      m_nSelection = TabCtrl_GetCurSel(m_hwnd);
-
-
-  return TRUE;
-}
-
 // remove one page from the notebook, without deleting
-bool wxNotebook::RemovePage(int nPage)
+wxNotebookPage *wxNotebook::DoRemovePage(int nPage)
 {
-  wxCHECK_MSG( IS_VALID_PAGE(nPage), FALSE, wxT("notebook page out of range") );
+    wxNotebookPage *pageRemoved = wxNotebookBase::DoRemovePage(nPage);
+    if ( !pageRemoved )
+        return NULL;
 
-  TabCtrl_DeleteItem(m_hwnd, nPage);
+    TabCtrl_DeleteItem(m_hwnd, nPage);
 
-  m_aPages.Remove(nPage);
+    if ( m_pages.IsEmpty() )
+    {
+        // no selection any more, the notebook becamse empty
+        m_nSelection = -1;
+    }
+    else // notebook still not empty
+    {
+        int selNew = TabCtrl_GetCurSel(m_hwnd);
+        if (selNew != -1)
+        {
+            // No selection change, just refresh the current selection.
+            // Because it could be that the slection index changed 
+            // we need to update it. 
+            // Note: this does not mean the selection it self changed.
+            m_nSelection = selNew;
+            m_pages[m_nSelection]->Refresh();
+        }
+        else if (int(nPage) == m_nSelection)
+        {
+            // The selection was deleted.
+            
+            // Determine new selection.
+            if (m_nSelection == int(GetPageCount()))
+                selNew = m_nSelection - 1;
+            else
+                selNew = m_nSelection;
+            
+            // m_nSelection must be always valid so reset it before calling
+            // SetSelection()
+            m_nSelection = -1;
+            SetSelection(selNew);
+        }
+        else
+        {
+            wxFAIL; // Windows did not behave ok.
+        }
+    }
 
-  if ( m_aPages.IsEmpty() )
-      m_nSelection = -1;
-    else
-      m_nSelection = TabCtrl_GetCurSel(m_hwnd);
-
-  return TRUE;
+    return pageRemoved;
 }
 
 // remove all pages
@@ -365,9 +398,9 @@ bool wxNotebook::DeleteAllPages()
   int nPageCount = GetPageCount();
   int nPage;
   for ( nPage = 0; nPage < nPageCount; nPage++ )
-    delete m_aPages[nPage];
+    delete m_pages[nPage];
 
-  m_aPages.Clear();
+  m_pages.Clear();
 
   TabCtrl_DeleteAllItems(m_hwnd);
 
@@ -395,6 +428,18 @@ bool wxNotebook::InsertPage(int nPage,
   wxASSERT( pPage != NULL );
   wxCHECK( IS_VALID_PAGE(nPage) || nPage == GetPageCount(), FALSE );
 
+#if wxUSE_UXTHEME && wxUSE_UXTHEME_AUTO
+    // Automatically apply the theme background,
+    // changing the colour of the panel to match the
+    // tab page colour. This won't work well with all
+    // themes but it's a start.
+    if (wxUxThemeEngine::Get() && pPage->IsKindOf(CLASSINFO(wxPanel)))
+    {
+        wxNotebookApplyThemeBackground(this, pPage, wxNotebookGetThemeBackgroundColour(this));
+    }
+#endif
+
+    // add a new tab to the control
   // do add the tab to the control
 
   // init all fields to 0
@@ -428,7 +473,7 @@ bool wxNotebook::InsertPage(int nPage,
   }
 
   // save the pointer to the page
-  m_aPages.Insert(pPage, nPage);
+  m_pages.Insert(pPage, nPage);
 
   // don't show pages by default (we'll need to adjust their size first)
   HWND hwnd = GetWinHwnd(pPage);
@@ -473,9 +518,9 @@ void wxNotebook::OnSize(wxSizeEvent& event)
 
   int width = rc.right - rc.left,
       height = rc.bottom - rc.top;
-  size_t nCount = m_aPages.Count();
+  size_t nCount = m_pages.Count();
   for ( size_t nPage = 0; nPage < nCount; nPage++ ) {
-    wxNotebookPage *pPage = m_aPages[nPage];
+    wxNotebookPage *pPage = m_pages[nPage];
     pPage->SetSize(rc.left, rc.top, width, height);
   }
 
@@ -489,12 +534,12 @@ void wxNotebook::OnSelChange(wxNotebookEvent& event)
   {
       int sel = event.GetOldSelection();
       if ( sel != -1 )
-        m_aPages[sel]->Show(FALSE);
+        m_pages[sel]->Show(FALSE);
 
       sel = event.GetSelection();
       if ( sel != -1 )
       {
-        wxNotebookPage *pPage = m_aPages[sel];
+        wxNotebookPage *pPage = m_pages[sel];
         pPage->Show(TRUE);
         pPage->SetFocus();
       }
@@ -515,7 +560,7 @@ void wxNotebook::OnSetFocus(wxFocusEvent& event)
 
     // set focus to the currently selected page if any
     if ( m_nSelection != -1 )
-        m_aPages[m_nSelection]->SetFocus();
+        m_pages[m_nSelection]->SetFocus();
 
     event.Skip();
 }
@@ -551,7 +596,7 @@ void wxNotebook::OnNavigationKey(wxNavigationKeyEvent& event)
                 // and is being propagated downwards
                 event.SetEventObject(this);
 
-                wxWindow *page = m_aPages[m_nSelection];
+                wxWindow *page = m_pages[m_nSelection];
                 if ( !page->GetEventHandler()->ProcessEvent(event) )
                 {
                     page->SetFocus();
@@ -592,6 +637,21 @@ bool wxNotebook::DoPhase(int WXUNUSED(nPhase))
   return TRUE;
 }
 
+// ----------------------------------------------------------------------------
+// wxNotebook Windows message handlers
+// ----------------------------------------------------------------------------
+
+bool wxNotebook::MSWOnScroll(int orientation, WXWORD nSBCode,
+                             WXWORD pos, WXHWND control)
+{
+    // don't generate EVT_SCROLLWIN events for the WM_SCROLLs coming from the
+    // up-down control
+    if ( control )
+        return FALSE;
+
+    return wxNotebookBase::MSWOnScroll(orientation, nSBCode, pos, control);
+}
+
 bool wxNotebook::MSWOnNotify(int idCtrl, WXLPARAM lParam, WXLPARAM* result)
 {
   wxNotebookEvent event(wxEVT_NULL, m_windowId);
@@ -620,42 +680,85 @@ bool wxNotebook::MSWOnNotify(int idCtrl, WXLPARAM lParam, WXLPARAM* result)
   return processed;
 }
 
-// ----------------------------------------------------------------------------
-// wxNotebook helper functions
-// ----------------------------------------------------------------------------
-
-// generate the page changing and changed events, hide the currently active
-// panel and show the new one
-void wxNotebook::ChangePage(int nOldSel, int nSel)
+// Windows only: attempts to get colour for UX theme page background
+wxColour wxNotebookGetThemeBackgroundColour(wxNotebook* notebook)
 {
-  // MT-FIXME should use a real semaphore
-  static bool s_bInsideChangePage = FALSE;
+#if wxUSE_UXTHEME
+    if (wxUxThemeEngine::Get())
+    {
+        HTHEME hTheme = wxUxThemeEngine::Get()->m_pfnOpenThemeData((HWND) notebook->GetHWND(), L"TAB");
+        if (hTheme)
+        {
+            // This is total guesswork.
+            // See PlatformSDK\Include\Tmschema.h for values
+            COLORREF themeColor;
+            wxUxThemeEngine::Get()->
+                m_pfnGetThemeColor(hTheme,
+                10 /* TABP_BODY */,
+                1 /* NORMAL */,
+                3821 /* FILLCOLORHINT */,
+                &themeColor);
 
-  // when we call ProcessEvent(), our own OnSelChange() is called which calls
-  // this function - break the infinite loop
-  if ( s_bInsideChangePage )
-    return;
+            /*
+            [DS] Workaround for WindowBlinds:
+            Some themes return a near black theme color using FILLCOLORHINT,
+            this makes notebook pages have an ugly black background and makes
+            text (usually black) unreadable. Retry again with FILLCOLOR.
 
-  // it's not an error (the message may be generated by the tab control itself)
-  // and it may happen - just do nothing
-  if ( nSel == nOldSel )
-    return;
+            This workaround potentially breaks appearance of some themes,
+            but in practice it already fixes some themes.
+            */
+            if (themeColor == 1)
+            {
+                wxUxThemeEngine::Get()->m_pfnGetThemeColor(hTheme,
+                    10 /* TABP_BODY */,
+                    1 /* NORMAL */,
+                    3802 /* FILLCOLOR */,
+                    &themeColor);
+            }
 
-  s_bInsideChangePage = TRUE;
-
-  wxNotebookEvent event(wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGING, m_windowId);
-  event.SetSelection(nSel);
-  event.SetOldSelection(nOldSel);
-  event.SetEventObject(this);
-  if ( ProcessEvent(event) && !event.IsAllowed() )
-  {
-    // program doesn't allow the page change
-    s_bInsideChangePage = FALSE;
-    return;
-  }
-
-  event.SetEventType(wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGED);
-  ProcessEvent(event);
-
-  s_bInsideChangePage = FALSE;
+            wxUxThemeEngine::Get()->m_pfnCloseThemeData(hTheme);
+            
+            wxColour colour(GetRValue(themeColor), GetGValue(themeColor), GetBValue(themeColor));
+            return colour;
+        }
+    }
+#endif
+    return notebook->GetBackgroundColour();
 }
+
+// Windows only: attempts to apply the UX theme page background to this page
+void wxNotebookApplyThemeBackground(wxNotebook* notebook, wxWindow* window, const wxColour& colour)
+{
+#if wxUSE_UXTHEME
+    // Don't set the background for buttons since this will
+    // switch it into ownerdraw mode
+    if (window->IsKindOf(CLASSINFO(wxButton)) && !window->IsKindOf(CLASSINFO(wxBitmapButton)))
+        // This is essential, otherwise you'll see dark grey
+        // corners in the buttons.
+        ((wxButton*)window)->wxControl::SetBackgroundColour(colour);
+    else if (window->IsKindOf(CLASSINFO(wxStaticText)) ||
+             window->IsKindOf(CLASSINFO(wxStaticBox)) ||
+             window->IsKindOf(CLASSINFO(wxStaticLine)) ||
+             window->IsKindOf(CLASSINFO(wxRadioButton)) ||
+             window->IsKindOf(CLASSINFO(wxRadioBox)) ||
+             window->IsKindOf(CLASSINFO(wxCheckBox)) ||
+             window->IsKindOf(CLASSINFO(wxBitmapButton)) ||
+             window->IsKindOf(CLASSINFO(wxSlider)) ||
+             window->IsKindOf(CLASSINFO(wxPanel)) ||
+             (window->IsKindOf(CLASSINFO(wxNotebook)) && (window != notebook)) ||
+             window->IsKindOf(CLASSINFO(wxScrolledWindow))
+        )
+    {
+        window->SetBackgroundColour(colour);
+    }
+
+    for ( wxWindowList::Node *node = window->GetChildren().GetFirst(); node; node = node->GetNext() )
+    {
+        wxWindow *child = node->GetData();
+        wxNotebookApplyThemeBackground(notebook, child, colour);
+    }
+#endif
+}
+
+#endif // wxUSE_NOTEBOOK
