@@ -17,8 +17,14 @@ def runTest(frame, nb, log):
 
 
 
-
-
-
 overview = """\
 """
+
+
+
+
+
+if __name__ == '__main__':
+    import sys,os
+    import run
+    run.main(['', os.path.basename(sys.argv[0])])
