@@ -1,8 +1,6 @@
 
 from wxPython.wx import *
 
-import string
-
 #----------------------------------------------------------------------
 
 class TestPanel(wxPanel):
@@ -17,6 +15,7 @@ class TestPanel(wxPanel):
         sc = wxSpinCtrl(self, -1, "", wxPoint(30, 50), wxSize(80, -1))
         sc.SetRange(1,100)
         sc.SetValue(5)
+        #sc.Enable(False)
 
 
 #----------------------------------------------------------------------
@@ -35,3 +34,12 @@ def runTest(frame, nb, log):
 
 overview = """\
 """
+
+
+
+
+
+if __name__ == '__main__':
+    import sys,os
+    import run
+    run.main(['', os.path.basename(sys.argv[0])])
